<?php
/**
 * @package			No Boss Extensions
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2020 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

defined("_JEXEC") or die;

class JFormFieldNobosshaslibrary extends JFormField
{
	/**
	 * The form field type.
	 *
	 * @var    string
	 * @since  3.2
	 */
	protected $type = "nobosshaslibrary";

    /**
	 * Method to get the field options.
	 *
	 * @return  array  The field option objects.
	 *
	 * @since   11.1
	 */
	protected function getInput(){  
        // Verifica se a pasta da library e do nobossajax existe
        $librayFolderExists = JFolder::exists(JPATH_LIBRARIES.'/noboss');
        $nobossAjaxFolderExists = JFolder::exists(JPATH_SITE.'/components/com_nobossajax');

        // Caso algum deles não esteja instalado
        if(!$librayFolderExists || !$nobossAjaxFolderExists) {
            // Inicia instalacao forcada da library
            try {
                // Adiciona diretorio de models do componente installer do Joomla
                JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_installer/models');
                // Instancia model de extensoes
                $modelInstaller = JModelLegacy::getInstance('install', 'InstallerModel');
                $input = JFactory::getApplication()->input;
                // Seta para utilizar metodo de instalacao via url
                $input->set('installtype', 'url');
                // Seta a url de instalacao da library
                $input->set('install_url', 'https://www.nobossextensions.com/en/installation/nobosslibrary/');
                // Executa metodo de instalacao
                $modelInstaller->install();

                if(!$nobossAjaxFolderExists){
                    // Seta a url de instalacao do noboss ajax
                    $input->set('install_url', 'https://www.nobossextensions.com/en/installation/nobossajax/');
                    // Executa metodo de instalacao
                    $modelInstaller->install();
                }
            } catch (Exception $e) {
                // Nao tem tratamento de erro pq o Joomla joga diversas vezes a falta de permissoes como erro sendo que ocorreu tudo certo
            }

            // Library foi instalada com sucesso
            if(JFolder::exists(JPATH_LIBRARIES.'/noboss')) {
                // Esconde o form para nao aparecer quebrado na tela
                echo '<style>.info-labels, .controls, .control-label{ display: none; } </style>';
                // Exibe mensagem que No Boss Library esta sendo atualizada e a pagina sera recarregada
                echo "<span style='text-align: center; font-size: 18px; width: 100%; position: absolute; font-weight: 600;'>The library used by No Boss Extensions is being updated... <br> Please wait for page to reload in 5 seconds.</span>";
                // Recarrega a pagina com a library ja instalada
                header("Refresh:5");
                exit;
            }

            // AQUI PARA BAIXO segue com modelo anterior que exiba erro na tela de library ou ajax nao
            $template = $this->form->getData()->get('template');  

            $formDataModule = $this->form->getData()->get('module');
            // Pega o nome do componente da url caso esteja nas configurações globais
            $componentNameGlobalConfig = JFactory::getApplication()->input->get->get('component');
            // Pega o nome da extensao 
            if (!empty($componentNameGlobalConfig)) {
                // pega o nome de nas configuraçoes globais
                $this->type = 'global';
            // Caso seja um template 
            } else if (!empty($template)) {
                // Pega a variavel template do form
                $this->type = 'template';
            }
            else if (empty($formDataModule)){
                // pega o nome de componentes
                $formNameArray = explode('.', $this->form->getName());
                $this->type = 'component';
            } 
            else {
                // Pega o nome em casos de modulo
                $this->type = 'module';
            }

            if($this->type == 'module'){
                // Cria um objeto vazio
                $params = new stdClass();
                // Substitui os parametros que estão para ser carregados por um objeto vazio (Ocorria um bug com campos com value json que tinham showon)
                $this->form->getData()->set('params', $params);
                
                // Busca o xml do form
                $xml = $this->form->getXml();
                $countFieldsets = count($xml->config->fields->fieldset);
                // Percorre cada fieldset
                for ($i=1; $i <= $countFieldsets; $i++) {
                    // Apaga o fieldset
                    unset($xml->config->fields->fieldset[$i]);
                }
            } else if ($this->type == 'component') {
                // Pega todos os dados
                $data = $this->form->getData();
                // Limpa cada um dos campos
                foreach($data->flatten() as $key => $item){
                    $data->set("$key", '');
                }

                $xml = $this->form->getXml();
                $countFieldsets = count($xml->fieldset);
                // Percorre cada fieldset
                for ($i=1; $i <= $countFieldsets; $i++) {
                    // Apaga o fieldset
                    unset($xml->fieldset[$i]);
                }
            }

            $html = '<div class="message-box col-lg-10 col-md-10 col-sm-10 col-xs-10">'.
                            '<span class="alert-icon fa fa-exclamation-triangle" aria-hidden="true"></span>'.
                            '<h3>Could not find the No Boss Library and/or No Boss Ajax</h3><br/>'.
                            '<p>Both are required for this extension to work. <br/> Reinstall the Library to resolve the issue.<br /> * Install the library on your Joomla site using the https://www.nobossextensions.com/en/installation/nobosslibrary/ link. <br />  * Notify No Boss support on the problem through the contact form https://www.nobossextensions.com/en/.</p>'.
                        '</div>';
            
            JFactory::getDocument()->addScriptDeclaration("
            jQuery(document).ready(function(){
                    var extraFields = jQuery('form').find('[name=task]').siblings('[type=hidden]').clone();
                
                    jQuery('form').html('{$html}');
                    jQuery('form').append('<input type=\"hidden\" name=\"task\" value=\"\" />');
                    jQuery('form').append(extraFields);

                    jQuery('#system-message-container').remove();
                    jQuery('.btn-toolbar').find('.btn-wrapper:not(#toolbar-cancel, #toolbar-help) button').prop('disabled', true);
                })
            ");

            JFactory::getDocument()->addStyleDeclaration("
                .message-box {
                    padding: 8em 5em;
                    font-size: 40px;
                    font-size: 0.9375em;
                    line-height: 1.2;
                    color: #777;
                    text-align: center;
                }

                .message-box .alert-icon {
                    font-size: 60px;
                }

                .message-box h3 {
                    line-height: 28px;
                    font-size: 30px;
                }

                .message-box p {
                    font-size: 20px;
                }
                
                .message-box {
                    background-color: #eee;
                    margin: auto;
                    float: none;
                    border-radius: 30px;
                }
            ");
            
        }
    }
}
