<?php
/**
 * @package			No Boss Extensions
 * @subpackage  	No Boss Calendar
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2020 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

defined("JPATH_PLATFORM") or die;
// verificacao necessaria, caso contrario um erro acontece na geracao de tema
if(!class_exists('JFormFieldNobosssubform')){
    require JPATH_LIBRARIES . '/noboss/forms/fields/nobosssubform.php'; 
}

class JFormFieldNobosssubformcalendar extends JFormFieldNobosssubform
{

    protected $type = "nobosssubformcalendar";

    protected function getInput(){
        // montagem do html dos filtros de ano e mes
        $html = "<div class='calendar-items-filter' data-id='items-filter' data-theme='{$this->fieldname}'>
            <p class='filters-title'>" . JText::_('MOD_NOBOSSCALENDAR_ITEMS_FILTERS_TITLE') . "</p>";
        
        // Ano atual
        $currentYear = (int)(new JDate())->year;
        // Mes atual
        $currentMonth = (int)(new JDate())->month;
        
        // campo select de ano
        $html .= "<div class='form-group'>
                    <span>" . JText::_('MOD_NOBOSSCALENDAR_ITEMS_FILTER_YEAR') . "</span>
                    <select class='form-control' data-id='year-filter'>";

        // Adiciona opcao de selecionar todos os anos
        $html .= "<option value='all'>" . JText::_('MOD_NOBOSSCALENDAR_ITEMS_FILTER_YEAR_ALL') . "</option>";
        
        // monta as options de anos (3 anos atras, ano atual e 3 anos a frente)
        for($i = $currentYear - 3; $i <= $currentYear + 3; $i++){
            $html .= "<option value='{$i}' " . ($i == $currentYear ? 'selected' : '') . ">{$i}</option>";
        }
        // fecha o select de anos
        $html .= "</select>
                </div>";
        
        // TODO: Se formos DESATIVAR opcao do mes vir selecionado, descontar codigo abaixo e comentar mais abaixo
        // campo select de mes, deixando selecionado o mes atual
        // $html .= "<div class='form-group'>
        //             <span>" . JText::_('MOD_NOBOSSCALENDAR_ITEMS_FILTER_MONTH') . "</span>
        //             <select class='form-control' data-id='month-filter'>
        //                 <option value=''>" . JText::_('MOD_NOBOSSCALENDAR_ITEMS_FILTER_MONTH_DEFAULT_OPT') . "</option>
        //                 <option value='all'>" . JText::_('MOD_NOBOSSCALENDAR_ITEMS_FILTER_MONTH_ALL') . "</option>
        //                 <option value='01'>" . JText::_('JANUARY') . "</option>
        //                 <option value='02'>" . JText::_('FEBRUARY') . "</option>
        //                 <option value='03'>" . JText::_('MARCH') . "</option>
        //                 <option value='04'>" . JText::_('APRIL') . "</option>
        //                 <option value='05'>" . JText::_('MAY') . "</option>
        //                 <option value='06'>" . JText::_('JUNE') . "</option>
        //                 <option value='07'>" . JText::_('JULY') . "</option>
        //                 <option value='08'>" . JText::_('AUGUST') . "</option>
        //                 <option value='09'>" . JText::_('SEPTEMBER') . "</option>
        //                 <option value='10'>" . JText::_('OCTOBER') . "</option>
        //                 <option value='11'>" . JText::_('NOVEMBER') . "</option>
        //                 <option value='12'>" . JText::_('DECEMBER') . "</option>
        //             </select>
        //         </div>";

        $html .= "<div class='form-group'>
                    <span>" . JText::_('MOD_NOBOSSCALENDAR_ITEMS_FILTER_MONTH') . "</span>
                    <select class='form-control' data-id='month-filter'>
                        <option value=''>" . JText::_('MOD_NOBOSSCALENDAR_ITEMS_FILTER_MONTH_DEFAULT_OPT') . "</option>
                        <option value='all'>" . JText::_('MOD_NOBOSSCALENDAR_ITEMS_FILTER_MONTH_ALL') . "</option>
                        <option value='01' " . (1 == $currentMonth ? 'selected' : '') . ">" . JText::_('JANUARY') . "</option>
                        <option value='02' " . (2 == $currentMonth ? 'selected' : '') . ">" . JText::_('FEBRUARY') . "</option>
                        <option value='03' " . (3 == $currentMonth ? 'selected' : '') . ">" . JText::_('MARCH') . "</option>
                        <option value='04' " . (4 == $currentMonth ? 'selected' : '') . ">" . JText::_('APRIL') . "</option>
                        <option value='05' " . (5 == $currentMonth ? 'selected' : '') . ">" . JText::_('MAY') . "</option>
                        <option value='06' " . (6 == $currentMonth ? 'selected' : '') . ">" . JText::_('JUNE') . "</option>
                        <option value='07' " . (7 == $currentMonth ? 'selected' : '') . ">" . JText::_('JULY') . "</option>
                        <option value='08' " . (8 == $currentMonth ? 'selected' : '') . ">" . JText::_('AUGUST') . "</option>
                        <option value='09' " . (9 == $currentMonth ? 'selected' : '') . ">" . JText::_('SEPTEMBER') . "</option>
                        <option value='10' " . (10 == $currentMonth ? 'selected' : '') . ">" . JText::_('OCTOBER') . "</option>
                        <option value='11' " . (11 == $currentMonth ? 'selected' : '') . ">" . JText::_('NOVEMBER') . "</option>
                        <option value='12' " . (12 == $currentMonth ? 'selected' : '') . ">" . JText::_('DECEMBER') . "</option>
                    </select>
                </div>";
                
        // avisos ao usuario
        $html .= "<div data-id='no-results-alert' class='alert alert-info'>" . JText::_('MOD_NOBOSSCALENDAR_NO_ITEMS_ALERT') . "</div>";
        $html .= "<div data-id='no-month-alert' class='alert alert-info'>" . JText::_('MOD_NOBOSSCALENDAR_NO_MONTH_SELECTED_ALERT') . "</div>";
        // fecha a div externa dos filtros
        $html .= "</div>";
        
        // valor a ser inserido no input hidden, contendo dados dos eventos
        $inputHiddenValue = json_encode($this->value);

        // Converte aspas simples para '##' para permitir carregamento em json (utilizado para versoes legadas antes do novo subform)
        $inputHiddenValue = str_replace("'", "##", $inputHiddenValue);

        // html do input hidden que vai guardar os eventos
        $html .= "<input data-id='calendar-input-hidden' name='{$this->name}' type='hidden' value='{$inputHiddenValue}' data-theme='{$this->fieldname}'/>";
        // limpa os itens para que eles nao sejam exibidos ao carregar a pagina
        $this->value = array();
        // chama o resto do html do nobosssubform
        $html .= parent::getInput();
        // Carrega o js e css do field personalizado
        $doc = JFactory::getDocument();
        $doc->addStylesheet(JURI::base()."../modules/mod_nobosscalendar/fields/assets/stylesheets/css/nobosssubformcalendar.min.css");
        $doc->addScript(JURI::base()."../modules/mod_nobosscalendar/fields/assets/js/min/nobosssubformcalendar.min.js");
        
        // adiciona constante no JS
        JText::script('MOD_NOBOSSCALENDAR_SUBFORM_COLLAPSE_TITLE_RECURRENT_INFO');
        JText::script('MOD_NOBOSSCALENDAR_EVENT_RECURRENCE_TYPE_OPT_WEEK_DAYS');
        JText::script('MOD_NOBOSSCALENDAR_EVENT_RECURRENCE_TYPE_OPT_SPECIFIC_DATES');
        JText::script('MOD_NOBOSSCALENDAR_EVENT_RECURRENCE_TYPE_OPT_SPECIFIC_DATES_HOURS');
        

        // Obtem dados do idioma que esta sendo navegadoi
        $lang = JFactory::getLanguage()->get('tag');
        // Obtem todos idiomas instalados
        $languages = JLanguageHelper::getLanguages('lang_code');

        // Language definido para navegacao no site: Obtem sef do idioma navegado (Ex: 'en', 'pt') para uso em url
        if(!empty($languages[$lang])){
            $sefLanguage = $languages[$lang]->sef;
        }
        // Pega o sef do idioma default
        else{
            //Pega parametros do componente "com_language"
            $paramsLanguage = JComponentHelper::getParams("com_languages");
            //Armazena idioma default do front-end.
            $defaultLanguage = $paramsLanguage->get("site");
            $sefLanguage = $languages[$defaultLanguage]->sef;
        }
        
        // Declara uma variavel JS com o sef do idioma para colocar em requisicoes ajax
        $doc->addScriptDeclaration("var lang = '{$sefLanguage}'");

        // exibe o field
        return $html;
    }
}
