<?php
/**
 * @package			No Boss Extensions
 * @subpackage  	No Boss Calendar
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2020 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

defined('_JEXEC') or die();

// Importa a library para conversão de url
jimport('noboss.util.url');
jimport('noboss.util.fonts');

/**
 * Helper do mod_nobosscalendar
 */
class ModNobosscalendarHelper {

	/**
	 * Método ajax que atualiza calendário conforme navegação entre meses (nao inclui modelo de cards)
	 *
	 * @return void.
	 */
	public static function showCalendarMonthAjax() {
        
        // Pega dados da requisição.
		$app = JFactory::getApplication();
		$input = $app->input;
		$isAjax = true;

		// Pega o id do módulo
		$idModule = $input->get("idModule");

		// Pega os parametros de estilo do mes do calendario
		$calendarParams = ModNobosscalendarHelper::getCalendarParams($idModule);
		$moduleParams = $calendarParams->moduleParams;
		$extensionName = $moduleParams->extension_name;	
		$moduleName = $moduleParams->module_name;	
		$theme = json_decode($moduleParams->theme)->theme;
		$itemsCustomization = $calendarParams->itemsCustomization;
		$externalArea = $calendarParams->externalArea;		

        // Cria datas para o calendário.
        $currentDate = JDate::getInstance($input->get("currentDate"));
		$slide = $input->get("slide");
		$date = static::getFirstDayMonth($currentDate, $slide);

		// Clona o primeiro dia para ser percorrido na montagem do calendario
		$dateFor = clone $date;

		// Pega data atual para para criar o calendário.
		$auxToday = new DateTime();
		$today = JDate::getInstance($auxToday->format('Y-m-d'));

		// Pega o filtro caso haja
		$filter = $input->get("filter");

		$isOpen = $input->get("isOpen", false) == 'true' ? true : false;

		// Dados para o calendário.
		$dataCalendar = self::getDataToCalendar($idModule, $date, $itemsCustomization, $filter);
        
        $lastDate = clone $date;
        $lastDate->modify('last day of this month');
        
		// Obtem eventos do mes
		// FIXME: comentado pq fui substutuido o uso pelo $dataCalendar
        //$eventsSearch = ModNobosscalendarHelper::getEvents($date, $lastDate, $idModule, $filter);

		$app = JFactory::getApplication();
		// pega o template vinculado a pagina
		$tmpl = $app->getTemplate(true);
		// pega os parametros do template
		$tmplParams = $tmpl->params;
		// pega a cor primaria setada nos parametros do template
        $tmplPrimaryColor = $tmplParams->get('primary_color');

        // Obtem parametros da modal
        $modalCustomization = ModNobosscalendarHelper::getModalParams($idModule);
		
		// Renderiza o html
		exit(require JModuleHelper::getLayoutPath('mod_nobosscalendar', 'common/calendar_inner'));
	}

	/**
	 * Método ajax que atualiza calendário conforme navegação entre meses (especifico do modelo de cards)
	 *
	 * @return void.
	 */
	public static function showCalendarMonthEventsCardsAjax() {
		// Pega dados da requisição.
		$app = JFactory::getApplication();
		$input = $app->input;

		// Pega o id do módulo
		$idModule = $input->get("idModule");

		// Pega os parametros do calendario
		$calendarParams = ModNobosscalendarHelper::getCalendarParams($idModule);
		$moduleParams = $calendarParams->moduleParams;
		$extensionName = $moduleParams->extension_name;	
		$moduleName = $moduleParams->module_name;	
		$theme = json_decode($moduleParams->theme)->theme;
		$itemsCustomization = $calendarParams->itemsCustomization;
		$externalArea = $calendarParams->externalArea;

		// Obtem info de qual botao foi clicado (prev / next)
		$slide = $input->get("slide");
        
		// Setado para nao exibir navegacao por mes
		if((isset($itemsCustomization->browsing_month)) && ($itemsCustomization->browsing_month == 0)){
            // Obtem data de corte (data do primeiro ou do ultimo elemento exibido, dependendo se navegacao eh 'prev' ou 'next')
            $courtDate = $input->get("courtDate");

			// Obtem eventos utilizando funcao que obtem eventos proximos da data atual
            $returnEventsCards = ModNobosscalendarHelper::getRunningEventsCards($courtDate, $idModule, $itemsCustomization, $slide);
			
			// Ha resultados a exibir
			if (!empty($returnEventsCards->events)){
                $eventsSearch = $returnEventsCards->events;
                unset($returnEventsCards->events);
                // Seta data inicial do primeiro evento como $date
				$date = new JDate($eventsSearch[0]->initial_date);
			}else{
                // retorna 0 para nao mudar nada na pagina
                exit(0);
            }
		}
		// Navegacao por mes
		else{
			// Cria datas para o calendário.
			$currentDate = JDate::getInstance($input->get("currentDate"));
			$date = static::getFirstDayMonth($currentDate, $slide);

			// Clona o primeiro dia para ser percorrido na montagem do calendario
			$dateFor = clone $date;

			$lastDate = clone $date;
			$lastDate->modify('last day of this month');
			
			// Obtem os eventos do mes navegado
			$eventsSearch = ModNobosscalendarHelper::getEvents($date, $lastDate, $idModule);
        }
               
        // Obtem parametros da modal
        $modalCustomization = ModNobosscalendarHelper::getModalParams($idModule);

		exit(require JModuleHelper::getLayoutPath('mod_nobosscalendar', 'theme/model2_body'));
    }
    
    /**
	 * Metodo que obtem os eventos a exibir no modelo 2 (cards) qnd setado para exibir eventos corridos (sem ser mes a mes)
	 *
	 * @param 	String 		$courtDate				Data de corte em formato de string (Y-d-m)
	 * @param 	Integer		$idModule 	 			Id do modulo
	 * @param 	object 		$itemsCustomization		Dados de customizacao do modulo
	 * @param 	String 		$navigation 	 		Sentido da navegacao do usuario (prev / next)
	 *
	 * @return 	object 		Array com eventos a exibir e array com infos sobre setas de navegacao
	 */
	public static function getRunningEventsCards($courtDate, $idModule, $itemsCustomization, $navigation = ''){
        $session = JFactory::getSession();

        // Monta objeto para retorno dos dados (vamos retornar os eventos e info de exibicao das setas)
        $return = new StdClass();

        // Converte para objeto date		
        $courtDate = self::createJDateFromString($courtDate, true);
	
		$date = clone $courtDate;
		$lastDate = clone $courtDate;

		// Seta data inicial de pesquisa
		$date->modify('-1 years');
		// Seta data final de pesquisa
		$lastDate->modify('+1 years');

		// Obtem eventos conforme criterio de pesquisa
		$events = ModNobosscalendarHelper::getEvents($date, $lastDate, $idModule);

		// Converte resultado para array
		$events = (array)$events;

    	// Array com eventos a exibir
        $eventsSearch = array();
        
        // Seta por padrao para setas de navegacao serem exibidas
        $return->show_navigation_prev = true;
        $return->show_navigation_next = true;

		// Exibicao em ordem crescente a partir da data informada
		if (($navigation == 'next') || ($navigation == '')){
            // Aumenta um dia no corte para nao dar problema qnd exibe soh um evento
            // FIXME: comentado pq estava excluindo eventos do dia atual
            //$courtDate->modify('+1 day');

			// Percorre todos eventos para identificar os a exibir a partir da data de corte
			foreach ($events as $i => $event) {
				// Obtem data inicial do evento
				$eventInitialDate = self::createJDateFromString($event->initial_date, true);

				// Data inicial eh igual ou maior que data de corte
				if ($eventInitialDate >= $courtDate){
					// Limite de eventos a exibir ainda nao foi atingido
					if (count($eventsSearch) < $itemsCustomization->events_to_display){
                        $eventsSearch[] = $event;
					}
					// Para foreach
					else{
						break;
					}
                }
			}

			// Limite de eventos a exibir ainda nao foi atingido: significa que nao tinha 'proximos eventos' que chega para exibir
			if (count($eventsSearch) < $itemsCustomization->events_to_display){
				// Inverte a ordem dos eventos para decrescente
				$events = array_reverse($events);

				// Percorre todos eventos para identificar os a exibir a partir da data de corte
				foreach ($events as $i => $event) {
					// Obtem data inicial do evento
					$eventInitialDate = self::createJDateFromString($event->initial_date, true);
					// Data inicial eh igual ou maior que data de corte
					if ($eventInitialDate < $courtDate){
						// Limite de eventos a exibir ainda nao foi atingido
						if (count($eventsSearch) < $itemsCustomization->events_to_display){
							$eventsSearch[] = $event;
						}
						// Para foreach
						else{
							break;
						}
					}
				}
            }
		}
		// Exibicao em ordem decrescente a partir da data informada (clicado para esquerda na navegacao)
		else{
            // Reduz um dia no corte para nao dar problema qnd exibe soh um evento
            $courtDate->modify('-1 day');

			// Ordena em forma decrescente os eventos a exibir
			uasort($events, array('self', 'sortDescendingEvents'));             

			// Percorre todos eventos para identificar os a exibir a partir da data de corte
			foreach ($events as $i => $event) {
				// Obtem data inicial do evento
				$eventInitialDate = self::createJDateFromString($event->initial_date, true);

				// Data inicial eh igual ou maior que data de corte
				if ($eventInitialDate <= $courtDate){
                    // Limite de eventos a exibir ainda nao foi atingido
					if (count($eventsSearch) < $itemsCustomization->events_to_display){
						$eventsSearch[] = $event;
					}
					// Para foreach
					else{
						break;
					}
                }
			}

			// Limite de eventos a exibir ainda nao foi atingido: significa que nao tinha 'eventos anteriores' que chega a exibir
			if (count($eventsSearch) < $itemsCustomization->events_to_display){
				// Inverte a ordem dos eventos para decrescente
				$events = array_reverse($events);

				// Percorre todos eventos para identificar os a exibir a partir da data de corte
				foreach ($events as $i => $event) {
					// Obtem data inicial do evento
					$eventInitialDate = self::createJDateFromString($event->initial_date, true);
					// Data inicial eh igual ou maior que data de corte
					if ($eventInitialDate > $courtDate){
						// Limite de eventos a exibir ainda nao foi atingido
						if (count($eventsSearch) < $itemsCustomization->events_to_display){
							$eventsSearch[] = $event;
						}
						// Para foreach
						else{
                            break;
						}
					}
				}
            }
        }
        
		// Ordena em forma crescente os eventos a exibir
        uasort($eventsSearch, array('self', 'sortAscendingEvents'));

        // Eventos a retornar
        $return->events = $eventsSearch;
        
        // Carregamento default (sem usuario clicar em setas de navegacao): nestes casos a busca traz todos eventos em $events
        if($navigation == ''){
            // Ordena em forma crescente os eventos
            uasort($events, array('self', 'sortAscendingEvents'));

            // Primeiro evento possivel de chegar na navegacao
            $eventFirst = array_shift($events);
            // Ultimo evento possivel de chegar na navegacao
            $eventLast = array_pop($events);

            // Armazena o id e data inicial do primeiro evento
            $session->set("event_first_{$idModule}", (object) array('id_event' => $eventFirst->id_event, 'initial_date' => $eventFirst->initial_date));
            // Armazena o id e data inicial do ultimo evento
            $session->set("event_last_{$idModule}", (object) array('id_event' => $eventLast->id_event, 'initial_date' => $eventLast->initial_date));
        }
        else{
            // Obtem da secao o primeiro evento possivel de exibir
            $eventFirst = $session->get("event_first_{$idModule}");
            // Obtem da secao o ultimo evento possivel de exibir
            $eventLast = $session->get("event_last_{$idModule}");
        }

        // Primeiro evento a exibir da consulta
        $eventFirstDisplay = array_shift($eventsSearch);
        // Ultimo evento a exibir da consulta
        $eventLastDisplay = array_pop($eventsSearch);

        // Somente um item sera exibido por vez
        if ($itemsCustomization->events_to_display == 1){
            // Seta o ultimo evento igual ao primeiro
            $eventLastDisplay = $eventFirstDisplay;
        }

        
        // Ultimo item a exibir tem mesmo ID e DATA do ultimo evento que pode ser exibido: esconde seta da direita
        if (($eventFirstDisplay->id_event == $eventFirst->id_event) && (self::createJDateFromString($eventFirstDisplay->initial_date, true) == self::createJDateFromString($eventFirst->initial_date, true))){
            $return->show_navigation_prev = false;
        }

        // Ultimo item a exibir tem mesmo ID e DATA do ultimo evento que pode ser exibido: esconde seta da direita
        if (($eventLastDisplay->id_event == $eventLast->id_event) && (self::createJDateFromString($eventLastDisplay->initial_date, true) == self::createJDateFromString($eventLast->initial_date, true))){
            $return->show_navigation_next = false;
        }

		return $return;
	}

	/**
	 * Método ajax que atualiza calendário conforme navegação entre meses.
	 * Recebe reqiosições ajax 
	 *
	 * @return void.
	 */
	public static function showCalendarYearMonthsEventsAjax() {
		// Pega dados da requisição.
		$app = JFactory::getApplication();
		$input = $app->input;
		// Pega o id do módulo
		$idModule = $input->get("idModule");
		// Pega os parametros de estilo do mes do calendario
		$calendarParams = ModNobosscalendarHelper::getCalendarParams($idModule);
		$itemsCustomization = $calendarParams->itemsCustomization;
		$externalArea = $calendarParams->externalArea;
        
		// Cria datas para o calendário.
		$currentDate = JDate::getInstance($input->get("currentDate"));
		// Pega a informação se avança ou retrocede
		$slide = $input->get("slide");
		// pega o primeiro dia do ano
		$date = self::getFirstDayOfYear($currentDate, $slide);
		// pega um array com todos os meses do ano e seu respectivo numero de eventos
		$yearMonths = self::getEventsYear($currentDate, $idModule, $slide);
		// Clona o primeiro dia para ser percorrido na montagem do calendario
		$dateFor = clone $date;
		exit(require JModuleHelper::getLayoutPath('mod_nobosscalendar', 'theme/model1_sidebar'));
	}
	
	/**
	 * Método que pega todas as parametrizações do calendário que são usadas por partes recarregáveis por ajax.
	 * Pega parametros da parte do mês do calendario e da listagem de evantos 
	 *
	 * @return Object Objeto com todos os parametros
	 */
	public static function getCalendarParams($idModule){

        $app = JFactory::getApplication();
        // pega o template vinculado a pagina
        $tmpl = $app->getTemplate(true);
        // pega os parametros do template
        $tmplParams = $tmpl->params;
        
        // pega a cor primaria setada nos parametros do template
        $tmplPrimaryColor = $tmplParams->get('primary_color');
        
        // pega a cor secundaria setada nos parametros do template
        $tmplSecondaryColor = $tmplParams->get('secondary_color');

		$dataModule = self::getDataModule($idModule);
		$itemsCustomization = (object) self::getItemsCustomization($idModule);	

		//variaveis necessarias para os arquivos de estilo
		$moduleParams = new StdClass();
		$moduleParams->extension_name = $dataModule->module;
		$moduleParams->module_name = str_replace("mod_", "", $dataModule->module);
		$moduleParams->id_module = $idModule;
		$moduleParams->theme = $dataModule->params->theme;
		$theme = json_decode($moduleParams->theme)->theme;
        
        // Largura maxima permitida no calendario
        if (!empty($itemsCustomization->calendar_maximum_width)){
            $itemsCustomization->calendar_maximum_width = "max-width: {$itemsCustomization->calendar_maximum_width}px";
        }
        else{
            $itemsCustomization->calendar_maximum_width = "";
        }

        if(!empty($itemsCustomization->calendar_font)){
			$itemsCustomization->calendar_font_family = NoBossUtilFonts::importNobossfontlist($itemsCustomization->calendar_font);
		}

		if(isset($itemsCustomization->sidebar_background_color)){
			$itemsCustomization->sidebar_background_color = !empty($itemsCustomization->sidebar_background_color) ? $itemsCustomization->sidebar_background_color : $tmplPrimaryColor;
		}
		
        // Cor de fundo da listagem de eventos
        $itemsCustomization->calendar_events_background_color = !empty($itemsCustomization->calendar_events_background_color) ? $itemsCustomization->calendar_events_background_color : $tmplPrimaryColor;
        $itemsCustomization->calendar_events_background_color = !empty($itemsCustomization->calendar_events_background_color) ? "background-color: {$itemsCustomization->calendar_events_background_color};" : "";

        // pega cor de fundo do cabecalho no mobile (caso nao esteja definido, pega a cor de fundo definida para lista de eventos)
        $itemsCustomization->month_header_mobile_background_color = !empty($itemsCustomization->month_header_mobile_background_color) ? "background-color: {$itemsCustomization->month_header_mobile_background_color};" : $itemsCustomization->calendar_events_background_color;
        
        // pega a cor definida no modulo ou a cor primaria do template
        $itemsCustomization->calendar_texts_color = !empty($itemsCustomization->calendar_texts_color) ? $itemsCustomization->calendar_texts_color : $tmplPrimaryColor;
        $itemsCustomization->calendar_texts_color = !empty($itemsCustomization->calendar_texts_color) ? "color: {$itemsCustomization->calendar_texts_color};" : "";

        // Cor de fundo da area do body do calendario
        if (!empty($itemsCustomization->calendar_background_color)){
            $itemsCustomization->calendar_background_color = "background-color: {$itemsCustomization->calendar_background_color};";
        }
        else{
            // Se nao estiver definido, pega cor da area de listagem
            $itemsCustomization->calendar_background_color = $itemsCustomization->calendar_events_background_color;
        }
        
        if(($theme == "model1") || ($theme == "model3")){
            // Font size para dias da semana no body
            if(empty($itemsCustomization->days_week_size)){
                $itemsCustomization->days_week_size = '16';
            }
            $itemsCustomization->days_week_size_em = $itemsCustomization->days_week_size/16;
            $itemsCustomization->days_week_size = "font-size: {$itemsCustomization->days_week_size}px; font-size: {$itemsCustomization->days_week_size_em}em;";

            // Font size mobile para dias da semana no body
            if(empty($itemsCustomization->days_week_size_mobile)){
                $itemsCustomization->days_week_size_mobile = '12';
            }
            $itemsCustomization->days_week_size_mobile_em = $itemsCustomization->days_week_size_mobile/16;
            $itemsCustomization->days_week_size_mobile = "font-size: {$itemsCustomization->days_week_size_mobile}px !important; font-size: {$itemsCustomization->days_week_size_mobile_em}em !important;";

            // Font size para dias do mês no body
            if(!empty($itemsCustomization->days_month_size)){
                $itemsCustomization->days_month_size_em = $itemsCustomization->days_month_size/16;
                $itemsCustomization->days_month_size = "font-size: {$itemsCustomization->days_month_size}px; font-size: {$itemsCustomization->days_month_size_em}em;";
            }else{
                $itemsCustomization->days_month_size = '';
            }

            // Font size mobile para dias do mês no body
            if(empty($itemsCustomization->days_month_size_mobile)){
                $itemsCustomization->days_month_size_mobile = '16';
            }
            $itemsCustomization->days_month_size_mobile_em = $itemsCustomization->days_month_size_mobile/16;
            $itemsCustomization->days_month_size_mobile = "font-size: {$itemsCustomization->days_month_size_mobile}px !important; font-size: {$itemsCustomization->days_month_size_mobile_em}em !important;";

            // Font size para filtro de categorias
            if(!empty($itemsCustomization->categories_size)){
                $itemsCustomization->categories_size_em = $itemsCustomization->categories_size/16;
                $itemsCustomization->categories_size = "font-size: {$itemsCustomization->categories_size}px; font-size: {$itemsCustomization->categories_size_em}em;";
            }
            else{
                $itemsCustomization->categories_size = '';
            }

            // Font size mobile para filtro de categorias
            if(!empty($itemsCustomization->categories_size_mobile)){
                $itemsCustomization->categories_size_mobile_em = $itemsCustomization->categories_size_mobile/16;
                $itemsCustomization->categories_size_mobile = "font-size: {$itemsCustomization->categories_size_mobile}px !important; font-size: {$itemsCustomization->categories_size_mobile_em}em !important;";
            }else{
                $itemsCustomization->categories_size_mobile = '';
            }
        }

		if($theme == "model1"){
            // Define valor default para tag html do ano no sidebar
            if (empty($itemsCustomization->year_header_tag_html)){
                $itemsCustomization->year_header_tag_html = 'h3';
            }

            // Font size para ano do sidebar
            if(empty($itemsCustomization->year_header_size)){
                $itemsCustomization->year_header_size = '29';
            }
            $itemsCustomization->year_header_size_em = $itemsCustomization->year_header_size/16;
            $itemsCustomization->year_header_size = "font-size: {$itemsCustomization->year_header_size}px; font-size: {$itemsCustomization->year_header_size_em}em;";

            // Font size mobile para ano do sidebar
            if(empty($itemsCustomization->year_header_size_mobile)){
                $itemsCustomization->year_header_size_mobile = '29';
            }
            $itemsCustomization->year_header_size_mobile = "font-size: {$itemsCustomization->year_header_size_mobile}px !important;";
            
            // Espacamento para ano no sidebar
            if (empty($itemsCustomization->year_header_spacing)){
                $itemsCustomization->year_header_spacing = "padding: 1.5em 1em 1em 1em;";
            }
            else{
                $itemsCustomization->year_header_spacing = "padding: " . implode(' ', $itemsCustomization->year_header_spacing) . ";";
            }

            // Espacamento para ano no sidebar para mobile
             if (empty($itemsCustomization->year_header_spacing_mobile)){
                $itemsCustomization->year_header_spacing_mobile = "padding: 0 1em 0 !important;";
            }
            else{
                $itemsCustomization->year_header_spacing_mobile = "padding: " . implode(' ', $itemsCustomization->year_header_spacing_mobile) . " !important;";
            }

            // Font size para meses do sidebar
            if(empty($itemsCustomization->sidebar_months_size)){
                $itemsCustomization->sidebar_months_size = '15';
            }
            $itemsCustomization->sidebar_months_size_em = $itemsCustomization->sidebar_months_size/16;
            $itemsCustomization->sidebar_months_size = "font-size: {$itemsCustomization->sidebar_months_size}px; font-size: {$itemsCustomization->sidebar_months_size_em}em;";
            
            // Font size mobile para meses do sidebar
            if(empty($itemsCustomization->sidebar_months_size_mobile)){
                $itemsCustomization->sidebar_months_size_mobile = '19';
            }
            $itemsCustomization->sidebar_months_size_mobile_em = $itemsCustomization->sidebar_months_size_mobile/16;
            $itemsCustomization->sidebar_months_size_mobile = "font-size: {$itemsCustomization->sidebar_months_size_mobile}px !important;; font-size: {$itemsCustomization->sidebar_months_size_mobile_em}em !important;";
            
            // Espacamento para meses no sidebar
            if (empty($itemsCustomization->sidebar_months_spacing)){
                $itemsCustomization->sidebar_months_spacing = "padding: 9px 72px 10px 16px;";
            }
            else{
                $itemsCustomization->sidebar_months_spacing = "padding: " . implode(' ', $itemsCustomization->sidebar_months_spacing) . ";";
            }

            // Espacamento para meses no sidebar para mobile
             if (empty($itemsCustomization->sidebar_months_spacing_mobile)){
                $itemsCustomization->sidebar_months_spacing_mobile = "padding: 10px 15px !important;";
            }
            else{
                $itemsCustomization->sidebar_months_spacing_mobile = "padding: " . implode(' ', $itemsCustomization->sidebar_months_spacing_mobile) . " !important;";
            }
		}		
		
		// Setas
		$itemsCustomization->arrows_style = "";
		$itemsCustomization->arrows_style_hover = "";
		$itemsCustomization->arrows_style_mobile = "";
		if(!empty($itemsCustomization->arrows_icon_size)){
			$arrowsIconSizeEm = $itemsCustomization->arrows_icon_size/16;
			$itemsCustomization->arrows_style = "font-size: {$itemsCustomization->arrows_icon_size}px; font-size: {$arrowsIconSizeEm}em;";
			if(isset($itemsCustomization->arrows_icon_size_mobile)){
				$arrowIconSizeMobile = $itemsCustomization->arrows_icon_size_mobile;
				$arrowIconSizeMobileEm = $arrowIconSizeMobile/16;
				$itemsCustomization->arrows_style_mobile =  "font-size: {$arrowIconSizeMobile}px !important; font-size: {$arrowIconSizeMobileEm}em !important;";
			}
		}
		
        // Cor das setas de navegacao (se nao estiver definido, pega cor primaria do template)
        $itemsCustomization->arrows_color = !empty($itemsCustomization->arrows_color) ? $itemsCustomization->arrows_color : $tmplPrimaryColor;
        $itemsCustomization->arrows_style .= " color: {$itemsCustomization->arrows_color};";

        // Cor das setas de navegacao no mobile (se nao estiver definido, pega do desktop)
        $itemsCustomization->arrows_color_mobile = !empty($itemsCustomization->arrows_color_mobile) ? $itemsCustomization->arrows_color_mobile : $itemsCustomization->arrows_color;
        $itemsCustomization->arrows_style_mobile .= " color: {$itemsCustomization->arrows_color_mobile} !important;";

        if ($itemsCustomization->show_arrows_border && !empty($itemsCustomization->arrows_border_color)){
            $itemsCustomization->arrows_style .= "border: 1px solid {$itemsCustomization->arrows_border_color};";
        }
        if ($itemsCustomization->show_arrows_border_radius && !empty($itemsCustomization->arrows_border_radius)){
            $itemsCustomization->arrows_style .= "border-radius: {$itemsCustomization->arrows_border_radius}px;";
        }
        if ($itemsCustomization->show_arrows_background && !empty($itemsCustomization->arrows_background)){
            // pega a cor definida no modulo ou a cor primaria do template
            $itemsCustomization->arrows_background = !empty($itemsCustomization->arrows_background) ? $itemsCustomization->arrows_background : $tmplPrimaryColor;
            $itemsCustomization->arrows_style .= "background-color: {$itemsCustomization->arrows_background}; ";
            // pega a cor definida no modulo ou a cor secundaria do template
            $itemsCustomization->arrows_background_hover = !empty($itemsCustomization->arrows_background_hover) ? $itemsCustomization->arrows_background_hover : $tmplSecondaryColor;
            $itemsCustomization->arrows_style_hover .= "background-color: {$itemsCustomization->arrows_background_hover} !important; ";
        }
        $itemsCustomization->arrows_style .= "padding: " . implode(' ', $itemsCustomization->arrows_spacing) . ";";
		
		$itemsCustomization->description = "";
		$itemsCustomization->month_header_style = "";

        // Modelo de cards
        if($theme == 'model2'){
			$itemsCustomization->month_header_font_family = NoBossUtilFonts::importNobossfontlist($itemsCustomization->month_header_font);
            $itemsCustomization->month_header_style .= $itemsCustomization->month_header_font_family;
            
            // Campo 'definir largura maxima para calendario' (sim / nao)
            if (empty($itemsCustomization->limit_maximum_width)){
                $itemsCustomization->limit_maximum_width = '0';
            }

            // Nao eh navegacao por meses e acesso eh mobile: forca para exibir apenas um item por vez
            if (($itemsCustomization->browsing_month == 0) && ((strpos($_SERVER['HTTP_USER_AGENT'],"iPhone") == true) || (strpos($_SERVER['HTTP_USER_AGENT'],"Android") == true))){
                $itemsCustomization->events_to_display = 1;
            }
            // Acesso tablet: exibe itens por linha conforme parametro proprio do tablet
            else if ((strpos($_SERVER['HTTP_USER_AGENT'],"iPad") == true) || (strpos($_SERVER['HTTP_USER_AGENT'],"Kindle") == true) || (strpos($_SERVER['HTTP_USER_AGENT'],"wOSBrowser") == true)){ 
                if (empty($itemsCustomization->events_per_line_tablet)){
                    $itemsCustomization->events_per_line_tablet = '2';
                }
                $itemsCustomization->events_per_line = $itemsCustomization->events_per_line_tablet;
            }
		}

		// string com o nome da modal da área externa concatenado com o modelo selecionado 
		$externalAreaXml = $theme . '_external_area_customization';
		// variável que armazena o objeto com os parâmetros da modal de área externa do tema selecionado  
		$externalArea = json_decode($dataModule->params->$externalAreaXml);
		//pega o valor da cor de fundo da area externa
		$externalArea = new StdClass();
		if(!empty($externalArea->external_area_background_color)){
            // pega a cor definida no modulo ou a cor primaria do template
            $externalArea->external_area_background_color = !empty($externalArea->external_area_background_color) ? $externalArea->external_area_background_color : $tmplPrimaryColor;
			$externalArea->external_area_background_color = $externalArea->external_area_background_color;
		}
		
        // Parametros para o mes no cabecalho
        $itemsCustomization->month_header_color = !empty($itemsCustomization->month_header_color) ? $itemsCustomization->month_header_color : $tmplPrimaryColor;
		$itemsCustomization->month_header_style .= !empty($itemsCustomization->month_header_color) ? "color: {$itemsCustomization->month_header_color};" : "";
		$itemsCustomization->month_header_style .= !empty($itemsCustomization->month_header_alignment) ? "text-align: {$itemsCustomization->month_header_alignment};" : "text-alignment: left;";
        $itemsCustomization->month_header_style .= !empty($itemsCustomization->month_header_transform) ? "text-transform: {$itemsCustomization->month_header_transform}; " : "text-transform: none; ";
        $itemsCustomization->month_header_style .= !empty($itemsCustomization->month_header_font) ? str_replace("\"", "'", NoBossUtilFonts::importNobossfontlist($itemsCustomization->month_header_font)) : "";

        // Espacamento do header de mes
        if(!empty($itemsCustomization->month_header_space)){
			$itemsCustomization->month_header_space = " padding:".implode(" ", $itemsCustomization->month_header_space).";";
        }else{
            $itemsCustomization->month_header_spac = '';
        }

        // Espacamento do header de mes no mobile
        if(!empty($itemsCustomization->month_header_space_mobile)){
			$itemsCustomization->month_header_space_mobile = "padding:".implode(" ", $itemsCustomization->month_header_space_mobile)." !important;";
        }else{
            $itemsCustomization->month_header_space_mobile = '';
        }

        // Font size para tamanho do mes no body
		if(!empty($itemsCustomization->month_header_size)){
			$itemsCustomization->month_header_size_em = $itemsCustomization->month_header_size/16;
			$itemsCustomization->month_header_size = "font-size: {$itemsCustomization->month_header_size}px; font-size: {$itemsCustomization->month_header_size_em}em;";
			$itemsCustomization->month_header_style .= $itemsCustomization->month_header_size;
		}
        
        // Font size mobile para tamanho do mes no body
		$itemsCustomization->month_header_style_mobile = "";
		if(!empty($itemsCustomization->month_header_size_mobile)){
			$monthHeaderSizeMobile = $itemsCustomization->month_header_size_mobile;
			$monthHeaderSizeMobileEm = $monthHeaderSizeMobile/16;
			$itemsCustomization->month_header_style_mobile .= "font-size: {$monthHeaderSizeMobile}px !important;";
		}

		// Busca os parametros da listagem de eventos
		// Personalização de itens 
		// Eventos por linha do calendário, funcional apenas para o modelo2
		if($theme == 'model2'){
            // Acesso desktop
            $itemsCustomization->min_width = "min-width: " . 100/$itemsCustomization->events_per_line . "%;";
            // Acesso mobile (soh pode um por linha)
            $itemsCustomization->min_width_mobile = "min-width: " . 100/1 . "% !important;";
            // Acesso tablet (possui campo proprio)
            $itemsCustomization->min_width_tablet = "min-width: " . 100/$itemsCustomization->events_per_line_tablet . "% !important;";
		}

		// Parâmetros para título
		$itemsCustomization->events_title_style = "";
		$itemsCustomization->events_title_font_family = NoBossUtilFonts::importNobossfontlist($itemsCustomization->events_title_font); 
		$itemsCustomization->events_title_style .= $itemsCustomization->events_title_font_family;
		$itemsCustomization->events_title_style .= !empty($itemsCustomization->events_title_alignment) ? "text-align: {$itemsCustomization->events_title_alignment}; " : "";
		$itemsCustomization->events_title_style .= !empty($itemsCustomization->events_title_transform) ? "text-transform: {$itemsCustomization->events_title_transform}; " : "";
		if(!empty($itemsCustomization->events_title_size)){
			$itemsCustomization->events_title_size_em = $itemsCustomization->events_title_size/16;
			$itemsCustomization->events_title_style .= "font-size: {$itemsCustomization->events_title_size}px; font-size: {$itemsCustomization->events_title_size_em}em;";
        }
        
		if(isset($itemsCustomization->events_title_size_mobile)){
			$eventsTitleSizeMobile = $itemsCustomization->events_title_size_mobile;
            $eventsTitleSizeMobileEm = $eventsTitleSizeMobile/16;
            
			$itemsCustomization->events_title_style_mobile =  "font-size: {$eventsTitleSizeMobile}px !important; font-size: {$eventsTitleSizeMobileEm}em !important;";
		}

		if(!empty($itemsCustomization->events_title_space)){
			$itemsCustomization->events_title_style .= " padding:".implode(" ", $itemsCustomization->events_title_space).";";
		}else{
            $itemsCustomization->events_title_style .= " padding:7px 0px 5px;";
        }

		// Parâmetros para texto de apoio
		$itemsCustomization->events_subtitle_style = "";
		$itemsCustomization->events_subtitle_font_family = NoBossUtilFonts::importNobossfontlist($itemsCustomization->events_subtitle_font);
		$itemsCustomization->events_subtitle_style .= NoBossUtilFonts::importNobossfontlist($itemsCustomization->events_subtitle_font);
		$itemsCustomization->events_subtitle_style .= !empty($itemsCustomization->events_subtitle_alignment) ? "text-align: {$itemsCustomization->events_subtitle_alignment}; " : "";
		$itemsCustomization->events_subtitle_style .= !empty($itemsCustomization->events_subtitle_transform) ? "text-transform: {$itemsCustomization->events_subtitle_transform}; " : "";
		
		if(!empty($itemsCustomization->events_subtitle_size)){
			$itemsCustomization->events_subtitle_size_em = $itemsCustomization->events_subtitle_size/16;
			$itemsCustomization->events_subtitle_style .= "font-size: {$itemsCustomization->events_subtitle_size}px; font-size: {$itemsCustomization->events_subtitle_size_em}em;";
		}
		if(isset($itemsCustomization->events_subtitle_size_mobile)){
			$eventsSubtitleSizeMobile = $itemsCustomization->events_subtitle_size_mobile;
			$eventsSubtitleSizeMobileEm = $eventsSubtitleSizeMobile/16;
			$itemsCustomization->events_subtitle_style_mobile =  "font-size: {$eventsSubtitleSizeMobile}px !important; font-size: {$eventsSubtitleSizeMobileEm}em !important;";
		}
		if(isset($itemsCustomization->events_subtitle_space)){
			$itemsCustomization->events_subtitle_style .= " padding:".implode(" ", $itemsCustomization->events_subtitle_space).";";
        }

        // Tamanho da fonte de horario
        if (empty($itemsCustomization->events_schedule_size)){
            $itemsCustomization->events_schedule_size = '12';
        }
        $itemsCustomization->events_schedule_size_em = $itemsCustomization->events_schedule_size/16;
		$itemsCustomization->events_schedule_style = "font-size: {$itemsCustomization->events_schedule_size}px; font-size: {$itemsCustomization->events_schedule_size_em}em;";

        // Tamanho da fonte de horario no mobile
		if(!empty($itemsCustomization->events_schedule_size_mobile)){
			$eventsScheduleSizeMobile = $itemsCustomization->events_schedule_size_mobile;
            $eventsScheduleSizeMobileEm = $eventsScheduleSizeMobile/16;
    		$itemsCustomization->events_schedule_style_mobile =  "font-size: {$eventsScheduleSizeMobile}px !important; font-size: {$eventsScheduleSizeMobileEm}em !important;";
        }
        else{
            $itemsCustomization->events_schedule_style_mobile = '';
        }

        // Espacamento do horario
        if (empty($itemsCustomization->events_schedule_space)){
            $itemsCustomization->events_schedule_style .= " padding: 0px 0px 0px;";
        }
		else{
			$itemsCustomization->events_schedule_style .= " padding:".implode(" ", $itemsCustomization->events_schedule_space).";";
        }
        
        // Tamanho da fonte de endereço
        if (empty($itemsCustomization->events_address_size)){
            $itemsCustomization->events_address_size = '12';
        }
        $itemsCustomization->events_address_size_em = $itemsCustomization->events_address_size/16;
        $itemsCustomization->events_address_style = "font-size: {$itemsCustomization->events_address_size}px; font-size: {$itemsCustomization->events_address_size_em}em;";
  
        // Tamanho da fonte de endereço no mobile
        if (empty($itemsCustomization->events_address_size_mobile)){
            $itemsCustomization->events_address_size_mobile = '12';
        }
        $eventsAddressSizeMobile = $itemsCustomization->events_address_size_mobile;
        $eventsAddressSizeMobileEm = $eventsAddressSizeMobile/16;
        $itemsCustomization->events_address_style_mobile =  "font-size: {$eventsAddressSizeMobile}px !important; font-size: {$eventsAddressSizeMobileEm}em !important;";

        // Espacamento do endereço
        if (empty($itemsCustomization->events_address_space)){
            $itemsCustomization->events_address_style .= " padding: 0px 0px; 5px;";
        }
		else{
			$itemsCustomization->events_address_style .= " padding:".implode(" ", $itemsCustomization->events_address_space).";";
        }

        // Cor para mensagem 'sem eventos a exibir'
        if (!empty($itemsCustomization->color_message_without_events)){
            $itemsCustomization->color_message_without_events = "color: {$itemsCustomization->color_message_without_events};";
        }else{
            $itemsCustomization->color_message_without_events = '';
        }
        
        // Parametro que define se barra lateral com lista de eventos pode ser exibida no desktop
        if (empty($itemsCustomization->display_month_events)){
            $itemsCustomization->display_month_events = '1';
        }

        // Largura minima a reservar para body do calendario (utilizado para saber se ha espaco para exibir barra lateral)
        if (!empty($itemsCustomization->bar_list_minimum_width)){
            // Divide por 100 para colocar em formato que permite calculos (Ex: 50% fica como 0.5)
            $itemsCustomization->bar_list_minimum_width = $itemsCustomization->bar_list_minimum_width / 100;
        }else{
            $itemsCustomization->bar_list_minimum_width = '0.5';
        }
        // Eventos a exibir por padrao na listagem lateral
        if (!isset($itemsCustomization->events_to_display_default)){
            $itemsCustomization->events_to_display_default = 'month';
        }

		$params = new StdClass();
		$params->moduleParams = $moduleParams;
		$params->itemsCustomization = $itemsCustomization;
		$params->externalArea = $externalArea;

		// Retorna um objeto com todos estes parametros
		return $params;
	}

	/**
	 * Pega os todos os meses do ano com o seu respectivo numero de eventos
	 *
	 * @param array $data			 Objeto data de referencia para saber o ano de busca
	 * @param JDate $idModule 	 	 id do múdulo para busca dos parametros
	 *
	 * @return array Retorna lista com os meses e o numero de eventos
	 */
	public static function getEventsYear($date, $idModule, $slide = 'current') {
		// Array de retorno
		$months = array();
		// Pega o primeiro mes do ano
		$date = self::getFirstDayOfYear($date, $slide);

		// Define o ano atual como limite
        $limitYear = $date->year;
        
		// Enquanto dentro do limite do ano da busca.
		while ($date->year == $limitYear) {
			// Data inicial do mes.
			$startDate = clone $date;
			$startDate->modify('first day of this month');
			$startDate->modify('midnight');
			// Data final do mes.
			$finalDate = clone $date;
			$finalDate->modify('last day of this month');
			$finalDate->modify('midnight');

            // Pega os eventos.
			$events = self::getEvents($startDate, $finalDate, $idModule);
			// Salva no array o mes como a chave e o numero de eventos como valor
			$months[$date->__toString()] = count((array)$events);
			// Avança uma mes
			$date = $date->getInstance($date->toSql() . " +1 month");
		}
		// Retorna o array
 		return $months;
	}

	/**
	 * Pega dias do calendario que possuem eventos ou períodos importantes.
	 *
	 * @param array $events				 Lista de eventos para extrair os dias e datas importantes.
	 * @param JDate $referenceDate 	 Data referência para buscar os dias de eventos.
	 *
	 * @return array Retorna lista com dias de eventos.
	 */
	public static function getEventsDays($events, $referenceDate = false) {

		$eventDays = array();

		// Data de hoje.
		$today = new DateTime();
		$jToday = new JDate($today->format('Y-m-d'));

		// Se é um calendário com o mesmo mês da data de hoje.
		if($referenceDate->month == $jToday->month) {
			/* Adiciona legenda de "hoje". */
			$legend['description'] = JText::_('MOD_NOBOSSCALENDAR_LEGEND_TODAY');
			$legend['class'] = 'square-hoje';
        }

        // Percorre todos os eventos.
		foreach ($events as $key => $event) {
            // Pega o primeiro dia do mes e define o mes de corte
			$date = self::getFirstDayMonth($referenceDate);
			$edgeMonth = $date->month;

			// Percorre todos os dias do mes do evento para saber em quais ele deve ser exibido
			while ($date->month == $edgeMonth) {
				// Pega a data inicial e final do evento/periodo
				$eventInitialDate = self::createJDateFromString($event->initial_date, true);
                $eventFinalDate = self::createJDateFromString($event->final_date, true);

                // Dia corrente esta nos dias do evento: adiciona ao array de retorno
                if (isset($event->days) && in_array($date->day, $event->days)){
                    $eventDays['days'][$date->day][] = $event;
                }
               
				// Avança um dia
				$date = $date->getInstance($date->toSql() . " +1 day");
            }
         }
        
 		return $eventDays;
	}

	/**
	 * Metodo que ordena eventos de forma crescente atraves da data informada
	 *
	 * @param array 	$calendarItem1		Evento 1 a comparar
	 * @param array 	$calendarItem2 		Evento 2 a comparar
	 *
	 * @return int Se a reunião 1 tiver data maior que a reunião 2 retorna 1, se a reunião 2
	 * tiver data maior que a reunião 1 retorna -1, se as duas datas forem iguais retorna 0.
	 */
	private static function sortAscendingEvents($calendarItem1, $calendarItem2) {
		// Cria datas para comparação do período final.
		$dateEvent1 = self::createJDateFromString($calendarItem1->initial_date, true);
		$dateEvent2 = self::createJDateFromString($calendarItem2->initial_date, true);

		if ($dateEvent1 == $dateEvent2) {
			return 0;
		} elseif ($dateEvent1 > $dateEvent2) {
			return 1;
		} else {
			return -1;
		}
	}

	/**
	 * Metodo que ordena eventos de forma decrescente atraves da data informada
	 *
	 * @param array 	$calendarItem1		Evento 1 a comparar
	 * @param array 	$calendarItem2 		Evento 2 a comparar
	 *
	 * @return int 
	 */
	private static function sortDescendingEvents($calendarItem1, $calendarItem2) {
		// Cria datas para comparação do período final.
		$dateEvent1 = self::createJDateFromString($calendarItem1->initial_date, true);
		$dateEvent2 = self::createJDateFromString($calendarItem2->initial_date, true);

		if ($dateEvent1 == $dateEvent2) {
			return 0;
		} elseif ($dateEvent1 < $dateEvent2) {
			return 1;
		} else {
			return -1;
		}
	}

	/**
	 * Pega dias de eventos e períodos importantes de uma faixa de data.
	 *
	 * @param  JDate 	$searchInicialData      Filtro de data incial
	 * @param  JDate 	$searchFinalDate        Filtro de data final
	 * @param  int		$idModule               Id do módulo.
	 *
	 * @return array 	Retorna uma lista de dados de objetos.
	 */
	public static function getEvents($searchInicialData, $searchFinalDate, $idModule, $filterCategory = false) {
        $app = JFactory::getApplication();
        // pega o template vinculado a pagina
        $tmpl = $app->getTemplate(true);
        // pega os parametros do template
        $tmplParams = $tmpl->params;
        // pega a cor primaria setada nos parametros do template
        $tmplPrimaryColor = $tmplParams->get('primary_color');
        // pega a cor secundaria setada nos parametros do template
        $tmplSecondaryColor = $tmplParams->get('secondary_color');
        
        // Pega dados do módulo.
		$dataModule = self::getDataModule($idModule);		
		$moduleParams = $dataModule->params;
        
		// Pega os eventos do subforma
		$itemsCustomization = self::getItemsCustomization($idModule);
		$theme = json_decode($moduleParams->theme)->theme;
        
        // Id do calendario definido
        if (isset($moduleParams->id_calendar)){
            $events = self::getEventsDataBase($moduleParams->id_calendar, 1);
        }

        // TODO: DEPRECATED: codigo abaixo serve para manter compatibilidade com usuarios que nao editaram o modulo desde o lancamento da versao com tabela de banco propria. Remover o codigo apos 08/2020
        // Nenhum evento localizado na base de dados: tenta encontrar no modo antigo (tabela params)
        if (empty($events)){
            $events = $moduleParams->{$theme};
            // trecho necessario para compatibilidade com modulos que estao salvos antes do update de filtros
            if(@json_decode($events)){
                // na versao nova um json eh recebido, entao devemos transforma-lo em um objeto php
                $events = json_decode($events);
            }
        }

		if(!empty($events)){
            // variavel que fica incrementando a cada evento sem categoria, para que os ids fiquem unicos
            $noneCounter = 1;

            // Converte resultado para array
            $events = (array)$events;

            // Total de eventos antes de percorre-los
            $totalEvents = count($events);

            // Contador do proximo item a criar de evento (se necessario criar novo item de evento) 
            $indexNextEvent = ($totalEvents+1);

            // Novo array com eventos
            $newEvents = array();

            // Percorre todas as datas configuradas no módulo.
            foreach ($events as $key => $event) {
                // Eventos salvos em tabela propria precisam ser decodados
                if(@json_decode($event)){
                    $event = json_decode($event);
                }

                // Seta valores default para campos que podem nao estar definidos
                if (!isset($event->recurrence_type)){
                    $event->recurrence_type = '';
                }
                if (!isset($event->link_target)){
                    $event->link_target = '';
                }
                if (!isset($event->event_subtitle)){
                    $event->event_subtitle = '';
                }
                if (!isset($event->event_place)){
                    $event->event_place = '';
                }
				if(!empty($event->recurrent_days)){
                    $event->recurrent_days = json_decode($event->recurrent_days);
                    // Converte para array e garante que as posicoes do array sejam int
                    $event->recurrent_days = array_values((array) $event->recurrent_days);
				}
				if(!empty($event->specific_dates)){
					$event->specific_dates = json_decode($event->specific_dates);
				}
				if(!empty($event->recurrent_days_ignore)){
					$event->recurrent_days_ignore = json_decode($event->recurrent_days_ignore);
				}
				else{
					$event->recurrent_days_ignore = array();
				}

                // Obtem data e hora atual
                $nowDate = date('Y-m-d H:i:s');

                // Data inicial do evento definida e evento nao eh recorrente do tipo datas especificadas
                if(!empty($event->initial_date) && (!isset($event->is_recurrent) || (($event->recurrence_type != 'specific-dates') && ($event->recurrence_type != 'specific-dates-hours')))){
                    
                    // Data final nao definida
                    if (empty($event->final_date)){
                        // Evento eh recorrente do tipo dia da semana: coloca data final como data atual somada em 5 anos (valor simbolico)
                        if ((isset($event->is_recurrent)) && ($event->is_recurrent)){
                            $event->final_date = (date('Y')+5).date('-m-d');
                        }
                        // Evento nao recorrente: seta data final igual a data inicial
                        else{
                            $event->final_date = $event->initial_date;
                        }
                    }

                    // Pega o objeto date a partir da string
					$eventInitialDate = self::createJDateFromString($event->initial_date, true);
					$eventFinalDate = self::createJDateFromString($event->final_date, true);

                    // ((data inicial evento < data inicial pesquisada) && (data final do evento < data inicial pesquisada) || (data inicial evento > data final pesquisada))
                    if((($eventInitialDate < $searchInicialData) && ($eventFinalDate < $searchInicialData)) || ($eventInitialDate > $searchFinalDate)){
                        continue;
                    }
                }

                // Decoda json das horas
                $event->hours = json_decode($event->hours);

                $category = new StdClass();

                // Categoria nao esta definida
                if (empty($event->event_category) || ($event->event_category == "none")){
                    $category->id = "none{$noneCounter}";
                    $category->legend = JText::_('MOD_NOBOSSCALENDAR_CATEGORY_COLOR');

                    // Cor ainda nao esta definida
                    if (empty($category->category_color)){
                        // Existe cor primaria do template nemesis
                        if(!empty($tmplPrimaryColor)){
                            $category->category_color = $tmplPrimaryColor;
                        }
                        // Cor manual
                        if (!empty($event->event_manual_color)){
                            $category->category_color = $event->event_manual_color;
                        }
                        // Seta uma cor default
                        else{
                            $category->category_color = '#333';
                        }
                    }

                    if($theme != "model2"){
                        $category->show_in_legends = $itemsCustomization->show_legends;
                    }
                    $event->event_category = $category;

                    // incrementa o contador de eventos sem categorias
                    $noneCounter++;
                }

                // Evento da categoria feriado
				else if($event->event_category == "holidays"){
 					$category = new StdClass();
					$category->id = $event->event_category;
                    $category->legend = JText::_('MOD_NOBOSSCALENDAR_CATEGORY_HOLIDAYS');
                    // pega a cor definida no modulo ou a cor primaria do template
                    $itemsCustomization->holidays_background_color = !empty($itemsCustomization->holidays_background_color) ? $itemsCustomization->holidays_background_color : $tmplPrimaryColor;
					$category->category_color = $itemsCustomization->holidays_background_color;
					
                    if($theme != "model2"){
                        $category->show_in_legends = $itemsCustomization->show_legends;
                    }
					
                    $event->event_category = $category;
                }
                // Tenta localizar categoria a partir de id que deve constar em $event->event_category
                else {
					// Troca o valor de categoria dentro do evento para o bjeto categoria
					$event->event_category = array_filter((array)$dataModule->params->categories, function($e) use ($event){
						return $e->id == $event->event_category;
                    });
                    
                    if(!empty($event->event_category)){
                        // Reseta o array para obter o primeiro objeto
                        $event->event_category = reset($event->event_category);
                    }
                }

                // Usuario filtrou uma categoria e evento nao pertence a ela: exclui do array
				if (!empty($filterCategory) && ($event->event_category->id != $filterCategory)) {
					continue;
                }

                // Converte ocorrencias de '##' (utilizado para nao ocorrer problemas com aspas ao salvar no admin) por aspas simples
				$event->event_title 	= str_replace("##", "'", $event->event_title);
				$event->event_subtitle 	= str_replace("##", "'", $event->event_subtitle);
                $event->event_place 	= str_replace("##", "'", $event->event_place);

                // Link para o evento
                switch ($event->link_option){
                    // Abrir item de menu do site
                    case 'internal':
                        if(!empty($event->link_internal)){
                            $event->link = NoBossUtilUrl::getUrlItemMenu($event->link_internal);
                        }else{
                            $event->link_option = 'none';
                        }
                        break;
                    // Abrir url externa
                    case 'external':
                        // Valida se a "URL externa" não é vazia e se é uma URL válida.
                        if(!empty($event->link_external) && filter_var($event->link_external, FILTER_VALIDATE_URL)){
                            $event->link = $event->link_external;
                        }else{
                            $event->link_option = 'none';
                        }
                        break;
                    // Abrir modal de conteudo
                    case 'modal':
						if(isset($event->modal_option)){
							// Tipo de modal
							switch ($event->modal_option){
								// Editor html
								case 'editor':
									$event->modal_content = str_replace("##", "'", $event->modal_content);
									break;
								// Artigo Joomla
								case 'article':
									if(!empty($event->modal_article)){
										$db = JFactory::getDBO();
										$sql = "SELECT introtext FROM #__content WHERE id = ".intval($event->modal_article);
										$db->setQuery($sql);
										$event->modal_content = $db->loadResult();
									}
									break;
								// Modulo
								case 'module':
									//$event->modal_module
									if(!empty($event->modal_module)){
                                        $version = new JVersion();

                                        // Versao do Joomla eh menor que 3.9: carrega o modulo pela posicao
                                        if ($version->RELEASE < '3.9') {
                                            // Obtem o conteudo a partir da posicao de modulo
                                            $db = JFactory::getDBO();
                                            $sql = "SELECT position FROM #__modules WHERE id = ".intval($event->modal_module);
                                            $db->setQuery($sql);
                                            $position = $db->loadResult();
                                            $event->modal_content = JHTML::_('content.prepare', '{loadposition '.$position.'}');
                                        }
                                        else{
                                            // Obtem o conteudo a partir do id
                                            $event->modal_content = JHTML::_('content.prepare', '{loadmoduleid '.$event->modal_module.'}');
                                        }
									}
									break;
							}
						}
                        break;
                }
                
                $event->link_target = $event->link_target ? "_blank" : "_self";	

                // Array com os dias que ocorre o evento
                $event->days = array();

                // Evento recorrente do tipo especificado por data: cria um novo evento para cada data
                if((isset($event->is_recurrent)) && ($event->is_recurrent) && (($event->recurrence_type == 'specific-dates') || ($event->recurrence_type == 'specific-dates-hours'))){
                    
                    // Datas nao estao especificadas
                    if (empty($event->specific_dates)){
                        continue;
                    }

                    // Percorre cada data especificada e gera um novo evento para cada uma
                    foreach($event->specific_dates as $keyDateEspecific => $dateEventEspecific){
                        // Converte data de string para objeto Date
                        $dateEventEspecific = self::createJDateFromString(($dateEventEspecific), true);
                        
                        // Data corrente eh menor que data inicial de pesquisa ou mais que data final de pesquisa
                        if(($dateEventEspecific < $searchInicialData) || ($dateEventEspecific > $searchFinalDate)){
                            continue;
                        }

                        // Clona objeto para modificar ele
                        $newEvent = clone $event;
                        // Coloca o dia atual como dia do evento
                        $newEvent->initial_date = $dateEventEspecific;
                        // Zera data final
                        $newEvent->final_date = $newEvent->initial_date;
                        // Remove informacao de que eh recorrente
                        $newEvent->is_recurrent = 0;
                        // Remove informacoes de dias a ignorar
                        $newEvent->recurrent_days_ignore = null;
                        // Adiciona o dia do evento no seu array de dias
                        $newEvent->days[] = (int)$dateEventEspecific->day;

                        /* Evento com datas e horas especificas: reseta o array de horas
                         * OBS: esse tipo de recorrencia utiliza os campos 'datas especificas' e 'horas' da tabela do banco onde
                         * as posicoes do array de cada campo desses corresponde a um conjunto de data e hora a exibir
                         */
                        if($event->recurrence_type == 'specific-dates-hours'){
                            $hour = new stdClass();

                            // Pega hora na mesma posicao da data percorrida
                            $hour->initial = $event->hours[$keyDateEspecific]->initial;
                            $hour->final = $event->hours[$keyDateEspecific]->final;

                            // Recria o array de horas deixando apenas a hora da data percorrida
                            $newEvent->hours = array();
                            $newEvent->hours[] = $hour;
                        }

                        // Adiciona como novo evento
                        $newEvents[] = $newEvent;
                        // Incrementa contador do proximo item a criar de evento
                        $indexNextEvent++;
                    }
                }

                // Evento ocorre em mais de um dia: percorre cada dia do evento para saber se eh um dia valido a exibir
                else if ($eventInitialDate != $eventFinalDate){
                    // Data inicial do evento eh menor que data inicial da pesquisa: seta data da pesquisa como data inicial a percorrer
                    if ($eventInitialDate < $searchInicialData){
                        $periodMonthInitial = clone $searchInicialData;
                    }else{
                        $periodMonthInitial = clone $eventInitialDate;
                    }
                    // Data final do evento eh maior que data final da pesquisa: seta data da pesquisa como data final a percorrer
                    if ($eventFinalDate > $searchFinalDate){
                        $periodMonthFinal = clone $searchFinalDate;
                    }else{
                        $periodMonthFinal = clone $eventFinalDate;
                    }
                    
                    // Percorre cada dia entre inicio e fim do periodo do evento no mes
                    while ($periodMonthInitial <= $periodMonthFinal){
                        $dateCurrent = clone $periodMonthInitial;

                        // ((Evento é recorrente) && (tipo de recorrencia eh por dia de semana) && (dia da semana atual nao esta marcado para o evento))
                        if ((isset($event->is_recurrent)) && ($event->is_recurrent) && ($event->recurrence_type == 'week-days') && (((array) $event->recurrent_days)[$dateCurrent->format('w')] === 0)){
                            // NAO ADICIONA DIA NO ARRAYS DE DIAS DO EVENTO
                        }
                        // Dia atual esta no array de 'recurrent_days_ignore'
                        else if(in_array($dateCurrent->format('Y/m/d'), (array) $event->recurrent_days_ignore)){
                            // NAO ADICIONA EVENTO PARA EXIBICAO
                        }
                        // Dia valido
                        else{
                            // Evento recorrente e modelo 2 (cards): forca que seja criado um evento novo para cada data
                            // TODO: se quisermos um dia deixar o evento recorrente de dia da semana ser duplicado para todos os modelos, basta remover a verificacao do campo theme abaixo que se aplicará a todos. A principal vantagem de mudar isso será o fato de que no modelo 1 marca ao lado do mes que tem somente um evento
                            if (((isset($event->is_recurrent)) && ($event->is_recurrent)) && ($theme == 'model2')){
                                // Clona objeto para modificar ele
                                $newEvent = clone $event;
                                // Coloca o dia atual como dia do evento
                                $newEvent->initial_date = $dateCurrent->format('Y-m-d');
                                // Zera data final
                                $newEvent->final_date = $newEvent->initial_date;
                                // Remove informacao de que eh recorrente
                                $newEvent->is_recurrent = 0;
                                // Remove informacoes de dias a ignorar
								$newEvent->recurrent_days_ignore = null;
                                // Adiciona o dia do evento no seu array de dias (nao eh usado no modelo de cards)
								$newEvent->days[] = (int)$dateCurrent->day;
                                // Adiciona como novo evento
                                $newEvents[] = $newEvent;
                                // Incrementa contador do proximo item a criar de evento
                                $indexNextEvent++;
                            }
                            // Mantem um evento unico para todas datas (nao eh usado no modelo de cards)
                            else{
                                $event->days[] = (int)$periodMonthInitial->day;
                            }                            
                        }

                        // Soma um dia
                        $periodMonthInitial->modify('+1 day');
                    }
                }
                // Evento de um dia: adiciona no array de dias do evento (nao eh usado no modelo de cards)
                else{
                    $event->days[] = (int)$eventInitialDate->day;
                }

                // Evento nao possui dias no mes atual: excluir do array
                if (empty($event->days)){
					continue;
                }
	
				// Caso as duas datas estejam vazias, remove o evento do array e segue o foreach para o proximo
				if(empty($event->initial_date) && empty($event->final_date)){
					continue;
                }
                $newEvents[] = $event;
            }
            
        }

		// Ordena eventos pelas suas datas.
		uasort($newEvents, array('self', 'sortAscendingEvents'));

		// Retorna os eventos como objeto
		return (object) $newEvents;
	}

	/**
	 * Pega dados de um módulo.
	 *
	 * @param type $idModule Id do módulo para recuperar dados.
	 *
	 * @return object Retorna dados do módulo.
	 */
	public static function getDataModule($idModule) {
		// Pega uma nova query.
		$dbo = JFactory::getDbo();
		$query = $dbo->getQuery(true);

		$query
			->select('*')
			->from('#__modules')
			->where('id = ' . (int) $idModule);

		$dbo->setQuery($query);
		$dataModule = $dbo->loadObject();

		// Realiza decode dos parâmetros.
		$dataModule->params = json_decode($dataModule->params);

		return $dataModule;
	}

	/**
	 * Pega data com o primeiro dia do mês.
	 *
	 * @param JDate $currentData Data atual referência.
	 * @param string $navigation (opcional) 'next' para pega o valor do mês seguinte
	 * ou 'prev' para o valor do mês anterior.
	 *
	 * @return JDate.
	 */
	public static function getFirstDayMonth($currentDate, $navigation = 'current') {

		$month = $currentDate->month;
		$year = $currentDate->year;

		// Verifica navegação do mês.
		switch ($navigation) {
			// Se mês seguinte.
			case 'next':
				$month = $currentDate->month + 1;
				// Se o próximo mês excedeu o mês 12(dezembro).
				if ($month == 13) {
					// Vai para janeiro do próximo ano.
					$month = 1;
					$year = $currentDate->year + 1;
				}
				break;

			// Se mês anterior.
			case 'prev':
				$month = $currentDate->month - 1;
				// Se o próximo mês limitou o mês 1(janeiro).
				if ($month == 0) {
					// Volta para dezembro do próximo passado.
					$month = 12;
					$year = $currentDate->year - 1;
				}
				break;

			default:
				break;
		}
		// Cria a data de retorno.
		return JDate::getInstance("{$year}-{$month}-01");
	}

	/**
	 * Retorna o ano da data passada, e de acordo com 
	 * o segundo parametro retorna o ano seguinte ou anterior
	 *
	 * @param JDate $currentData Data atual referência.
	 * @param string $navigation (opcional) 'next' para pega o valor do ano seguinte
	 * ou 'prev' para o valor do ano anterior.
	 *
	 * @return JDate.
	 */
	public static function getFirstDayOfYear($currentDate, $navigation = 'current') {
		$year = $currentDate->year;

		// Verifica navegação do ano.
		switch ($navigation) {
			// Se ano seguinte.
			case 'next':
				$year = $currentDate->year + 1;
				break;

			// Se ano anterior.
			case 'prev':
				$year = $currentDate->year - 1;
				break;

			default:
				break;
		}
		// Cria a data de retorno.
		return JDate::getInstance("{$year}-01-01");
	}


	/**
	 * Monta os dados para o calendario, retornando os dias do mes com seus respectivos eventos
	 *
	 * @param   int         $idModule 					Id do módulo atual
	 * @param   JDate       $dateCalendarReference		Objeto jdate com uma data do mes atual para referencia
     * @param   boolean     $itemsCustomization			Objeto com dados da modal de customizacao dos itens
	 * @param   boolean     $filter					    Filtro para os eventos
	 * @return  ArraY 	    Retorna os dias do mês com os eventos de cada um
	 */
	public static function getDataToCalendar($idModule, JDate $dateCalendarReference, $itemsCustomization, $filter = false) {
		$eventsDays = array();
		// Pega dados do módulo.
		$dataModule = self::getDataModule($idModule);
		// Data inicial do calendário.
		$startDate = clone $dateCalendarReference;
		$startDate->modify('first day of this month');
		$startDate->modify('midnight');
		// Data final do calendário.
		$finalDate = clone $dateCalendarReference;
		$finalDate->modify('last day of this month');
		$finalDate->modify('midnight');

		// Pega os eventos.
        $events = self::getEvents($startDate, $finalDate, $idModule);

		// Caso exista ao menos um evento
		if(empty($events)){
			return false;
		}
		// Cria a chave de legendas
        $eventsDays = array("legends" => array());

        // Monta o array com as legendas que existem nos eventos deste mes
        $eventsDays['legends'] = array_values(array_filter(array_map(function($a){
            if(!empty($a->event_category->id) && $a->event_category->id != 'none')
            return $a->event_category;
        }, (array)$events)));

        // Setado para exibir todos os filtros de categorias, mesmo que nao tenha eventos vinculados
        if (!empty($itemsCustomization->show_legends) && ($itemsCustomization->show_legends == 2)){
            // Obtem todas legendas do calendario
            $legends = (array)$dataModule->params->categories;

            // Concatena com as legendas do mes apenas para pegar feriados, caso tenha (depois eh removidos duplicados)
            $eventsDays['legends'] = array_merge($legends, $eventsDays['legends']);
        }

        // Limpa legendas repetidas
        $eventsDays['legends'] = array_filter($eventsDays['legends'],function($obj){
            static $idList = array();
            if(in_array($obj->id, $idList)) {
                return false;
            }
            $idList[] = $obj->id;
            return true;
        });

		// Filtra os eventos
		if(!empty($filter)){
			$events = array_filter((array)$events, function($a) use ($filter){ return $a->event_category->id == $filter;});
		}
		
        // Pega dia de cada evento ou periodo importante.
		$eventsDays = array_merge($eventsDays, self::getEventsDays($events, $dateCalendarReference));

		return $eventsDays;
    }

    /**
     * Metodo que pega as parametrizacoes para modais de conteudo
     *
     * @return Object Objeto com todos os parametros da modal de customizacao
     */
    public static function getModalParams($idModule, $modalCustomization = false){
        // Parametros da modal precisam ser buscados no banco (requisicao ajax)
        if (!$modalCustomization){
            // Obtem dados do modulo
            $dataModule = self::getDataModule($idModule);
            // Extrai parametros do modulo
            $params = $dataModule->params;
            $theme = json_decode($params->theme)->theme;
            // Obtem os parametros da modal de cusotomizacao
            $modalCustomization = json_decode($params->modal_customization);
        }

        // Parametros da modal nao definidos
        if (empty($modalCustomization)){
            /* TODO: manter o codigo abaixo apenas por um periodo para clientes que ja tinha registro de modulo criado antes do lancamento da versao 2.0. Remover apos 06/2020 e colocar apenas 'return false;'
            */
            // Seta valores default para a modal de personalizacao do conteudo
            $modalCustomization = json_decode('{"modal_title_font_select_font_list":"Roboto_Regular.ttf","modal_title_font":"{\"fontfamily\":\"Roboto_Regular.ttf\",\"externalLinked\":\"\",\"fontStyle\":\"Regular\"}","modal_title_font_font_external_url":"","modal_title_font_font_style":"Regular","modal_title_transform":"uppercase","modal_title_size":"30","modal_title_size_mobile":"30","modal_title_color":"#2c3e50","show_category":"1","modal_category_font_select_font_list":"Roboto_Regular.ttf","modal_category_font":"{\"fontfamily\":\"Roboto_Regular.ttf\",\"externalLinked\":\"\",\"fontStyle\":\"Regular\"}","modal_category_font_font_external_url":"","modal_category_font_font_style":"Regular","modal_category_transform":"none","modal_category_size":"16","modal_category_size_mobile":"16","show_display_background_color":"1","modal_category_color":"rgba(255, 255, 255, 1)","show_date":"1","show_hour":"1","show_address":"1","modal_details_font_select_font_list":"Roboto_Regular.ttf","modal_details_font":"{\"fontfamily\":\"Roboto_Regular.ttf\",\"externalLinked\":\"\",\"fontStyle\":\"Regular\"}","modal_details_font_font_external_url":"","modal_details_font_font_style":"Regular","modal_details_transform":"none","modal_details_size":"16","modal_details_size_mobile":"16","modal_details_color":"#2c3e50"}');

            //return false;
        }

        // Titulo da modal
        $modalCustomization->titleStyle = "";
        $modalCustomization->titleStyle .= NoBossUtilFonts::importNobossfontlist($modalCustomization->modal_title_font);
        $modalCustomization->titleStyle .= " text-transform: {$modalCustomization->modal_title_transform};";
        $modalCustomization->titleStyle .= " color: {$modalCustomization->modal_title_color};";
        $modalTitleSize = $modalCustomization->modal_title_size;
        if(!empty($modalTitleSize)){
            $modalTitleSizeEm = $modalTitleSize/16;
            $modalCustomization->titleStyle .= " font-size: {$modalTitleSize}px; font-size: {$modalTitleSizeEm}em;";
        }
        
        // Titulo da modal para mobile
        $modalCustomization->titleStyleMobile = "";
        if(isset($modalCustomization->modal_title_size_mobile)){
            $modalTitleSizeMobile  = $modalCustomization->modal_title_size_mobile;
            $modalTitleSizeMobileEm = $modalTitleSizeMobile/16;
            $modalCustomization->titleStyleMobile = " font-size: {$modalTitleSizeMobile}px; font-size: {$modalTitleSizeMobileEm}em !important;";
        }

        // Categoria
        if (!isset($modalCustomization->show_category)) { $modalCustomization->show_category = 1; }
        $modalCustomization->categoryStyle = "";
        if (!empty($modalCustomization->modal_category_font)){ $modalCustomization->categoryStyle .= NoBossUtilFonts::importNobossfontlist($modalCustomization->modal_category_font); }
        if (!empty($modalCustomization->modal_category_transform)){ $modalCustomization->categoryStyle .= " text-transform: {$modalCustomization->modal_category_transform};"; }
        if (!empty($modalCustomization->modal_category_color)){ $modalCustomization->categoryStyle .= " color: {$modalCustomization->modal_category_color};"; }
        if (!empty($modalCustomization->modal_category_size)){ $modalCategorySize = $modalCustomization->modal_category_size; }
        
        if(!empty($modalCategorySize)){
            $modalCategorySizeEm = $modalCategorySize/16;
            $modalCustomization->categoryStyle .= " font-size: {$modalCategorySize}px; font-size: {$modalCategorySizeEm}em;";
        }
        
        // Categoria para mobile
        $modalCustomization->categoryStyleMobile = "";
        if(isset($modalCustomization->modal_category_size_mobile)){
            $modalCategorySizeMobile  = $modalCustomization->modal_category_size_mobile;
            $modalCategorySizeMobileEm = $modalCategorySizeMobile/16;
            $modalCustomization->categoryStyleMobile = " font-size: {$modalCategorySizeMobile}px; font-size: {$modalCategorySizeMobileEm}em !important;";
        }
        
        // Detalhes do conteudo
        if (!isset($modalCustomization->show_date)) { $modalCustomization->show_date = 1; }
        if (!isset($modalCustomization->show_hour)) { $modalCustomization->show_hour = 1; }
        if (!isset($modalCustomization->show_address)) { $modalCustomization->show_address = 1; }
        $modalCustomization->detailsStyle = "";
        $modalCustomization->detailsStyle .= NoBossUtilFonts::importNobossfontlist($modalCustomization->modal_details_font);
        $modalCustomization->detailsStyle .= " text-transform: {$modalCustomization->modal_details_transform};";
        $modalCustomization->detailsStyle .= " color: {$modalCustomization->modal_details_color};";
        $modalDetailsSize = $modalCustomization->modal_details_size;
        if(!empty($modalDetailsSize)){
            $modalCustomization->detailsStyle .= " font-size: {$modalDetailsSize}px;";
        }
        
        // Detalhes do conteudo para mobile
        $modalCustomization->detailsStyleMobile = "";
        if(isset($modalCustomization->modal_details_size_mobile)){
            $modalDetailsSizeMobile  = $modalCustomization->modal_details_size_mobile;
            $modalCustomization->detailsStyleMobile = " font-size: {$modalDetailsSizeMobile}px !important;";
        }

        if (empty($modalCustomization->show_display_background_color)) {
            $modalCustomization->show_display_background_color = 'rgba(255, 255, 255, 1)';
        }

        if (empty($modalCustomization->show_address)) {
            $modalCustomization->show_address = 1;
        }
    
        return $modalCustomization;
    }

	/**
	 * Cria um objeto JDate dado um formato e uma string.
     * 
	 * @param string $stringDate 	String com a data a ser criada.
	 *
	 * @return JDate Retorna objeto criado.
	 */
	public static function createJDateFromString($stringDate, $clearTime = false) {
        // Obtem configuracoes globais
        $config     = JFactory::getConfig();
        
        // Obtem offset das configuracoes globais
        // OBS: fixado timezone de sao paulo para evitar problemas na conversao como ja ocorreu com fuso da finlandia que tem 6hrs a menos que o brasil
        $dateOffSet = 'America/Sao_Paulo'; 
        //$config->get('offset', 'America/Sao_Paulo');

        // Obtem objeto da data e hora
        $auxDate = JFactory::getDate($stringDate, $dateOffSet);
        // Converte a data para o formato desejado
        $auxDate->format('Y-m-d');

        $jDate = JDate::getInstance($auxDate->format('Y-m-d'));
		if($clearTime){
			$jDate->modify('midnight');
        }

		return $jDate;
	}

	/**
	 * Busca e extrai o objeto com a parametrização de itens
	 *
	 * @param int $idModule 		Id do módulo
	 *
	 * @return stdClass 			Retorna objeto
	 */
	public static function getItemsCustomization($idModule){
		// Pega os dados do módulo
		$module = self::getDataModule($idModule);
		$params = $module->params;
		$theme = json_decode($params->theme)->theme;
		
		// string com o nome da modal de itens concatenado com o modelo selecionado 
		$itemsCustomizationXml = $theme . '_items_customization';
		// variável que armazena o objeto com os parâmetros da modal de itens do tema selecionado  
		$itemsCustomization = json_decode($params->$itemsCustomizationXml);

		return $itemsCustomization;
    }
    
    /**
	 * Salva os eventos que estavam sendo editados via administrator (requisicao ajax na interceptacao do save)
	 *
	 * @return  void
	 */
    public static function saveEventsAjax(){
        error_reporting(0);

		$app = JFactory::getApplication();
        $input = $app->input;

		// Obtem parametros da requisicao
        $idModule = $input->get("id_module");
        $idCalendar = $input->get("id_calendar");
        $events = json_decode($input->get("data_json", '', 'RAW'));
        $eventsToRemove = $input->get("events_to_remove");
        
        $response = array();
		
        // Id do calendario nao definido: retorna mensagem de erro
        if (empty($idCalendar)){
			$response['success'] = false;
            $response['message'] = JText::_("MOD_NOBOSSCALENDAR_ID_CALENDAR_NOT_FOUND");
            exit(json_encode($response));
        }
		
        // Instancia banco de dados
        $db = JFactory::getDBO();

        if (count($events) > 0){
            // Percorre todos os eventos para salvar
            foreach ($events as $event){
                $event = (object)$event;
                $query = $db->getQuery(true);
                $idEvent = $event->id_event;
                $titleEvent = $event->event_title;
                $columns = array();
                $values = array();

                // Colunas do banco a salvar
                $columns[] = 'id_calendar';
                $columns[] = 'event_title';
                $columns[] = 'is_recurrent'; 
                $columns[] = 'recurrence_type';
                $columns[] = 'recurrent_days';
                $columns[] = 'specific_dates';
                $columns[] = 'recurrent_days_ignore';
                $columns[] = 'initial_date';
                $columns[] = 'final_date';
                $columns[] = 'hours';
                $columns[] = 'published';
                $columns[] = 'modal_content';
                
                // Valores a inserir nas colunas (COLOCAR NA MESMA ORDEM DAS COLUNAS ACIMA)
                $values[] = $db->quote("$idCalendar");
                $values[] = $db->quote("$event->event_title");
                $values[] = $db->quote("$event->is_recurrent");
                $values[] = $db->quote("$event->recurrence_type");
                // Dias da semana definidos
                if (!empty($event->recurrent_days)){
                    // Encoda em json os dias da semana
                    $event->recurrent_days = addslashes(json_encode($event->recurrent_days));
                }
                $values[] = $db->quote($event->recurrent_days);
                // Datas especificas definidas
                if (!empty($event->specific_dates)){
                    // Encoda em json as datas definidas
                    $event->specific_dates = addslashes(json_encode($event->specific_dates));
                }
                $values[] = $db->quote($event->specific_dates);
                // Datas a ignorar definidas
                if (!empty($event->recurrent_days_ignore)){
                    // Encoda em json as datas a ignorar
                    $event->recurrent_days_ignore = addslashes(json_encode($event->recurrent_days_ignore));
                }
                $values[] = $db->quote($event->recurrent_days_ignore);
                $values[] = $db->quote("$event->initial_date");
                $values[] = $db->quote("$event->final_date");
                $values[] = $db->quote("$event->hours");
                $values[] = $db->quote("$event->published");
                // Substitui aspas duplas por simples no conteudo (aspas duplas faz com que json quebre na hora de carregar os dados)
                $event->modal_content = str_replace("\"", "'", $event->modal_content);

                // Limpa quebra de pagina utilizando funcao 'chr' que identifica via ascii
                $event->event_title = str_replace(chr(012), "", $event->event_title);
                $event->event_subtitle = str_replace(chr(012), "", $event->event_subtitle);
                $event->event_place = str_replace(chr(012), "", $event->event_place);
                $event->modal_content = str_replace(chr(012), "", $event->modal_content);

                // Limpa '  ' (tab) utilizando funcao 'chr' que identifica via ascii
                $event->event_title = str_replace(chr(011), "", $event->event_title);
                $event->event_subtitle = str_replace(chr(011), "", $event->event_subtitle);
                $event->event_place = str_replace(chr(011), "", $event->event_place);
                $event->modal_content = str_replace(chr(011), "", $event->modal_content);

                $values[] = $db->quote("$event->modal_content");
                
                // Remove valores que serao inseridos em colunas proprias
                unset($event->id_event);
                unset($event->event_title);
                unset($event->is_recurrent);
                unset($event->recurrence_type);
                unset($event->recurrent_days);
                unset($event->specific_dates);
                unset($event->recurrent_days_ignore);
                unset($event->initial_date);
                unset($event->final_date);
                unset($event->hours);
                unset($event->published);
                unset($event->modal_content);
                
                // Salva os dados restantes do evento em formato json em coluna propria
                $columns[] = 'data_json';
                // TODO: avaliar utilizar a funcao json_last_error ou json_last_error_msg apos o json_encode para verificar eventuais erros (https://www.php.net/manual/pt_BR/function.json-last-error.php)
                $values[] = $db->quote(json_encode($event));

                // Id do modulo definido: ajusta para inserir junto
                if (!empty($idModule)){
                    $columns[] = 'id_module';
                    $values[] = $db->quote("$idModule");
                }

                // Define o tipo de cadastro como manual
                $columns[] = 'input_type';
                $values[] = $db->quote("manual");

                // Id do modulo nao definido (novo modulo) OU id do evento nao definido (edicao de modulo com novo evento)
                if (empty($idModule) || ($idModule == 0) || empty($idEvent) || ($idEvent == 0)){
                    // Realiza insert no banco
                    $query->insert($db->quoteName('#__noboss_calendar'))
                    ->columns($db->quoteName($columns))
                    ->values(implode($values, ','));
                }
                // Id do modulo definido: edicao de modulo
                else{
                    $fields = array();
                    // colunas a serem usadas na query de update
                    foreach ($columns as $key => $column) {
                        $fields[] = $db->quoteName($column) . ' = ' . $values[$key];
                    }
                    // condicoes a serem usadas no where da query
                    $conditions = array(
                        $db->quoteName('id_calendar') . ' = ' . $db->quote($idCalendar),
                        $db->quoteName('id_event') . ' = ' . (int) $idEvent
                    );
                    // Realiza update no banco
                    $query->update($db->quoteName('#__noboss_calendar'))->set($fields)->where($conditions);
                }

                // Seta query
                $db->setQuery($query);
                
                try{
                    // Executa query com sucesso
                    $db->execute();
                   
                }catch(Exception $e){
                    // seta flag de sem sucesso em caso de erro
                    $response['success'] = false;
                    $response['message'] = JText::sprintf('MOD_NOBOSSCALENDAR_SAVE_EVENT_DB_ERROR', $event->event_title);
                    exit(json_encode($response));
                }
            }
        }

        // Ha eventos para remover
        if (!empty($eventsToRemove)){
            try{
                self::removeEventsFromSpecifiedIds($eventsToRemove);
            }catch(Exception $e){
                // seta flag de sem sucesso em caso de erro
                $response['success'] = false;
                $response['message'] = JText::sprintf('MOD_NOBOSSCALENDAR_REMOVE_EVENTS_DB_ERROR', implode($eventsToRemove, ','));
                exit(json_encode($response));
            }
            
        }

        // Executa funcao de limpeza que remove eventos que nao possuem mais modulos
        self::removeEventsWithoutModule();

        if (!isset($response['success']) || $response['success'] == false){
            $response['success'] = true;
        }

        // Retorna a resposta em json
        exit(json_encode($response));
    }

    /**
	 * Obtem os eventos a editar em formato json (requisicao ajax no carregamento da edicao do modulo)
	 *
	 * @return  void
	 */
    public static function getEventJsonAjax(){
        error_reporting(0);

        $app = JFactory::getApplication();
        $input = $app->input;

		// Obtem parametros da requisicao
        $idCalendar = $input->get("id_calendar");

        try{
            $response['events'] = self::getEventsDataBase($idCalendar);
            $response['success'] = true;
            // Seta a mensagem para caso ocorra erro no JSON.parse realizado no JS apos o retorno desta funcao
            $response['error_json'] = JText::_('MOD_NOBOSSCALENDAR_GET_EVENT_JSON_PARSE_ERROR');
            // Seta a mensagem para caso ocorra algum outro erro no JS apos o retorno desta funcao
            $response['error_js'] = JText::_('MOD_NOBOSSCALENDAR_GET_EVENT_JAVASCRIPT_ERROR');
        }
        catch(Exception $e){
            // seta flag de sem sucesso em caso de erro
            $response['success'] = false;
            $response['error_database'] = $e->getMessage();
        }
        
        // Retorna a resposta em json
        exit(json_encode($response));
    }

    /**
	 * Obtem os eventos do banco de dados
     * 
     * @param int $idCalendar 			Id do calendario
	 * @param int $onlyPublished 		Somente eventos publicados
	 *
	 * @return  void
	 */
    public static function getEventsDataBase($idCalendar, $onlyPublished = 0){
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);

        // Realiza consulta no banco juntando colunas proprias com a coluna 'data_json' ficando tudo em um unico json
        /* 
         * TODO: a limpeza dos caracteres que dao problemas no json sao feitas na consulta sql abaixo e na funcao PHP que salva os dados
         *      - A limpeza esta sendo feita usando codigo ascii.
         *      - A tabela ascii pode ser acessada em http://www.asciitable.com/
         *          - Para consulta SQL utilizar o codigo da primeira coluna da tabela que eh 'Dec' utilizando a funcao CHAR do mysql
         *          - Para o PHP utilizar o codigo da coluna 'Oct' na funcao php chr
         */

         // Os dados sao todos agrupados em um unico json de retorno
        $query
        ->select("concat(\"{\", 
                    '\"id_event\":\"',id_event,'\",',
                    
                    /* Obtem o titulo convertendo aspas duplas para simples e removendo tab e quebra de linha */
                    '\"event_title\":\"', REPLACE(REPLACE(REPLACE(REPLACE(event_title, '\"', \"'\"),CHAR(10),''),CHAR(13),''),CHAR(9),''),'\",',
                    '\"is_recurrent\":\"',is_recurrent,'\",',
                    '\"recurrence_type\":\"',CASE WHEN (recurrence_type IS NULL) THEN '' ELSE TRIM(recurrence_type) END,'\",',
                    '\"recurrent_days\":\"',CASE WHEN (recurrent_days IS NULL) THEN '' ELSE TRIM(recurrent_days) END,'\",',
                    '\"specific_dates\":\"',CASE WHEN (specific_dates IS NULL) THEN '' ELSE TRIM(specific_dates) END,'\",',
                    '\"recurrent_days_ignore\":\"',CASE WHEN (recurrent_days_ignore IS NULL) THEN '' ELSE TRIM(recurrent_days_ignore) END,'\",',
                    '\"initial_date\":\"',CASE WHEN (initial_date = '0000-00-00') THEN '' ELSE initial_date END,'\",',
                    '\"final_date\":\"',CASE WHEN (final_date = '0000-00-00') THEN '' ELSE final_date END,'\",',
                    '\"hours\":\"',CASE WHEN (hours IS NULL) THEN '' ELSE TRIM(hours) END,'\",',
                    '\"published\":\"',published,'\",',
                    
                    /* Obtem o conteudo da modal convertendo aspas duplas para simples e removendo tab e quebra de linha */
                    '\"modal_content\":\"',CASE WHEN (modal_content IS NULL) THEN '' ELSE TRIM(REPLACE(REPLACE(REPLACE(REPLACE(modal_content, '\"', \"'\"),CHAR(10),''),CHAR(13),''),CHAR(9),'')) END,'\",',

                    /* Obtem demais dados do 'data_json' removendo tab, quebra de linha e '}' do final do json deste campo */
                    substring(TRIM(REPLACE(REPLACE(REPLACE(data_json,CHAR(10),''),CHAR(13),''),CHAR(9),'')), 2, length(data_json) - 2),'}') as data_json
                    ")
        ->from('#__noboss_calendar')
        ->where('id_calendar =' . $db->quote($idCalendar))
        ->where("input_type = 'manual'");

        if($onlyPublished){
            $query->where('published = "1"');
        }

        // Seta query
		$db->setQuery($query);
		
		//echo str_replace('#__', 'ext_', $query); exit;

        try{
            $events = $db->loadColumn();
        }
        // Ocorreu um erro na consulta
        catch(Exception $e){
            // Verifica se tabela existe no banco
            $query2 = $db->getQuery(true);
            $query2 = "SHOW TABLES LIKE '%_noboss_calendar'";
            $db->setQuery($query2);

            // Retorna erro que tabela nao existe
            if(empty($db->loadResult())){
                throw new Exception(JText::_('MOD_NOBOSSCALENDAR_GET_EVENT_DB_ERROR_NOT_FOUND_DATABASE'));
            }
            // Retorna erro da consulta FIXME: avaliar de exibirmos a consulta sql no erro
            else{
                // Obtem configuracoes globais
                $config     = JFactory::getConfig();
                throw new Exception(JText::sprintf('MOD_NOBOSSCALENDAR_GET_EVENT_DB_ERROR', $e->getMessage(),str_replace('#__', $config->get('dbprefix'), $query)));
            }

        }
        
        if (empty($events)){
            return false;
        }

        return $events;
    }

    /**
	 * Remove eventos que nao possuem mais vinculos com modulos (casos de modulos que sao excluidos)
	 *
	 * @return  Boolean     True ou false
	 */
    public static function removeEventsWithoutModule(){
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);

        $query
            ->delete('#__noboss_calendar')
            ->where("id_module IS NOT NULL
                        AND NOT EXISTS (
                            SELECT * FROM #__modules
                            WHERE id = id_module
                                AND module = 'mod_nobosscalendar')");

        $db->setQuery($query);

        // Executa query
        $db->execute();
    }

    /**
	 * Remove eventos de id's especificados
     * 
     * @param   Array      $idsEvents 		Ids dos eventos a remover
	 *
	 * @return  Boolean     True ou false
	 */
    public static function removeEventsFromSpecifiedIds($idsEvents){
        $db = JFactory::getDBO();
        $query = $db->getQuery(true);

        $idsEvents = "'".implode($idsEvents, "','")."'";

        $query
            ->delete('#__noboss_calendar')
            ->where("id_event IN ($idsEvents)");

        $db->setQuery($query);

        // Executa query
        $db->execute();
    }

    /**
	 * Verifica se No Boss Library esta instalada no site e caso nao esteja, forca uma atualizacao e recarrega a pagina
     * 
     * OBS: essa acao eh necessaria para contornar problema em atualizacoes da library no processo do Joomla que as vezes mantem ela removida
	 *
	 * @return  void
	 */
    public static function checkLibraryInstallation(){
        // Library da No Boss nao localizada: inicia instalacao forcada da library
        if(!JFolder::exists(JPATH_LIBRARIES.'/noboss')) {
            try {
                // Adiciona diretorio de models do componente installer do Joomla
                JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_installer/models');
                // Instancia model de extensoes
                $modelInstaller = JModelLegacy::getInstance('install', 'InstallerModel');
                $input = JFactory::getApplication()->input;
                // Seta para utilizar metodo de instalacao via url
                $input->set('installtype', 'url');
                // Seta a url de instalacao da library
                $input->set('install_url', 'https://www.nobossextensions.com/en/installation/nobosslibrary/');
                // Executa metodo de instalacao
                $modelInstaller->install();
            } catch (Exception $e) {
                // Nao tem tratamento de erro pq o Joomla joga diversas vezes a falta de permissoes como erro sendo que ocorreu tudo certo
            }

            // Library foi instalada com sucesso
            if(JFolder::exists(JPATH_LIBRARIES.'/noboss')) {
                // Recarrega a pagina
                header("Refresh:0");
                exit;
            }
            else{
                return false;
            }
        }
        return true;
    }

    /**
	 * Funcao executada antes de qualquer acao de update do joomla
     * 
	 * @param   JUpdate       $update  An update definition
     * @param   JTableUpdate  $table   The update instance from the database
	 */
    public static function prepareUpdate($update, $table){
        jimport('noboss.util.installscript');

        // Token da licenca
        $token = str_replace('token=', '', $update->get('extra_query'));

        if (method_exists(NoBossUtilInstallscript,'updateLicenseIsValid')){
            // Busca no servidor da No Boss se extensao possui permissao para update
            $return = NoBossUtilInstallscript::updateLicenseIsValid($token);
            
            // Token nao localizado
            if ($return == 'INVALID_TOKEN'){
                $msg = "<b>There are problems with the extension license:</b> <br /><br /> &bull; Your extension token was not found on the No Boss Extensions platform. <br /><br />";
                JFactory::getApplication()->enqueueMessage($msg, 'error');
                return false;
            }
            // Extensao nao possui suporte valido
            else if($return == 'INVALID'){
                $msg = "<b>License with expired update period:</b> <br /><br /> &bull; To update the extension, it is necessary to renew the license support period. <br /> &bull; You can renew the support time by accessing the tab called 'License' available within the edition of any extension registration. <br /><br />";
                JFactory::getApplication()->enqueueMessage($msg, 'error');
                return false;
            }
        }
    }
}
