<?php
/**
 * @package			No Boss Extensions
 * @subpackage  	No Boss Library
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2020 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

// Importa classe de instalacao da library
// TODO: trocado em 03/02/20 de jimport para require por causa de problemas em alguns sites que nao estavam reconhecendo a classe incluida no arquivo
//jimport('noboss.util.installscript');
require_once JPATH_SITE."/libraries/noboss/util/installscript.php";

class mod_NobosscalendarInstallerScript{

    /**
     * Array com todos os extras que foram instalados junto com a extensão
     * @var array
     */
    protected $extraExtensions = array('com_nobossajax', 'nobossautoupdate', 'noboss');

    /**
     * Evento pre-fligt do Joomla.Esse método é executado antes de qualquer outro, sendo instalação ou update
     * Esse é o unico momento onde pode cancelar a instalação
     *
     * @param   string     $type   Tipo de intalações (install, update, discover_install)
     * @param   JInstaller $parent Parent object
     *
     * @return  boolean  true caso deva ser instalada, false para cancelar a intalação
     */
    function preflight($type, $parent){
        $extraInfoPath = __DIR__ . '/extra-info.json';
        // Verifica se o arquivo json com o token e o plano existem
        if(JFile::exists($extraInfoPath)){
            // Lê o conteudo do json que contem o token
            $this->extraInfo = json_decode(file_get_contents($extraInfoPath), true);
            // Monta um array apenas com as informações que serão salvas no extra_query
            $extraQueryArray = array(
                'token' => $this->extraInfo['token']
            );

            // Chama a função que faz a ação no banco
            NoBossUtilInstallscript::updateExtraQuery($parent, http_build_query($extraQueryArray));
        }

        // Diretorios que precisam permissao de escrita
        $writingDirs = array();
        $writingDirs[] = JPATH_SITE.'/libraries/';
        $writingDirs[] = JPATH_SITE.'/layouts/';
        $writingDirs[] = JPATH_SITE.'/layouts/noboss/';
        $writingDirs[] = JPATH_SITE.'/modules/';
        $writingDirs[] = JPATH_SITE.'/components/';
        $writingDirs[] = JPATH_SITE.'/administrator/components/';
        $writingDirs[] = JPATH_SITE.'/plugins/system/';

        // Percorre todos diretorios para tentar alterar a permissao e gerar alerta de erro caso as permissoes nao possam ser alteradas
        foreach ($writingDirs as $dir){
            // Confirma se diretorio existe
            if(JFolder::exists($dir)){
               // Tenta alterar permissao do diretorio
                @chmod($dir, 0775); 
                // Diretorio esta sem permissao de escrita
                if (!is_writable($dir)) {
                    $msg = "The No Boss extension can not be installed or updated due to lack of write permissions in the '{$dir}' directory. Raise the permissions for the directory and internal files recursively and then try to install the extension again.";
                    JLog::add($msg, JLog::WARNING, 'error');
                    throw new RuntimeException($msg, 500);
                }
            }
        }
    }

    /**
     * Método  de desistalação  da  extensão
     *
     * @param   \JInstallerAdapterPackage   $parent é a classe chamando esse método
     *
     * @return void
     */
    function uninstall($parent){
        if(class_exists('NoBossUtilInstallscript')){
            // Valida se não existe mais alguma dependência alem da que esta sendo desinstalada
            if(NoBossUtilInstallscript::getDependencies($this->extraExtensions) <= 1){
                // Caso não tenha, desinstala todos os extras
                NoBossUtilInstallscript::uninstallExtras($this->extraExtensions);
            }

            // Nome da extensao (ex: mod_nobosscalendar)
            $extensionName = strtolower($parent->manifest->name);

            jimport('noboss.forms.fields.nobosslicense.nobosslicensemodel');
            // Obtem o token da base local
            $tokenArray = NobossModelNobosslicense::getLicenseTokenAndPlan($extensionName);
            // Cria propriedades no contexto para uso posterior
            $token = array_key_exists("token", $tokenArray) ? $tokenArray['token'] : '';

            if($token){
                // Executa funcao para remover url do site atual da base de dados de sites onde a extensao esta instalada
                NoBossUtilInstallscript::removeUrlSite($token);
            }
        }
    }

    /**
     * Metodo executao após o término da instalação/update
     */
    public function postflight($type, $parent) {
        // Caso seja uma instalação
        if($type == 'install'){
            // Manda a url do site que está sendo instalado para o servidor
            NoBossUtilInstallscript::saveAuthorizedUrl($this->extraInfo['token']);
        }

        // Eh Update
        if($type == 'update'){
            // Verifica se a library esta na lista de atualizacoes pendentes
            $libUpdateId = NoBossUtilInstallscript::getLibraryUpdateId($parent);

            // Library precisa ser atualizada
            if(!empty($libUpdateId)){
                // Adiciona diretorio de models do admin
                JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_installer/models');
                // Instancia model de extensoes
                $updateModel = JModelLegacy::getInstance('update', 'InstallerModel');
                // Realiza o update da library usando o id dela
                $updateModel->update(array($libUpdateId));
            }

            // TODO: remove arquivos que nao existem mais a partir da versao 2.0. Remover linhas abaixo apos Dezembro 2020
            jimport('joomla.filesystem.file');
            JFile::delete(JPATH_ROOT . '/modules/mod_nobosscalendar/subforms/model1_items_manual.xml');
            JFile::delete(JPATH_ROOT . '/modules/mod_nobosscalendar/subforms/model2_items_manual.xml');
            JFile::delete(JPATH_ROOT . '/modules/mod_nobosscalendar/subforms/model3_items_manual.xml');


            /*
                TODO: funcao temporaria para atualizar novo formato de cadastro de horarios para os eventos
                        * Executado a partir da versao 2.1 criada em Abril de 2020
                        * Remover apos Abril de 2021
            */
            $this->updateVersion2_1();
        }

        // Verifica se a pasta da library e do nobossajax existe
        $librayFolderExists = JFolder::exists(JPATH_LIBRARIES.'/noboss');
        $nobossAjaxFolderExists = JFolder::exists(JPATH_SITE.'/components/com_nobossajax');

        // Library nao foi instalada
        if(!$librayFolderExists) {
            // Inicia instalacao forcada da library
            try {
                // Adiciona diretorio de models do componente installer do Joomla
                JModelLegacy::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_installer/models');
                // Instancia model de extensoes
                $modelInstaller = JModelLegacy::getInstance('install', 'InstallerModel');
                $input = JFactory::getApplication()->input;
                // Seta para utilizar metodo de instalacao via url
                $input->set('installtype', 'url');
                // Seta a url de instalacao da library
                $input->set('install_url', 'https://www.nobossextensions.com/en/installation/nobosslibrary/');
                // Executa metodo de instalacao
                $modelInstaller->install();
            } catch (Exception $e) {
                // Nao tem tratamento de erro pq o Joomla joga diversas vezes a falta de permissoes como erro sendo que ocorreu tudo certo
            }

            // Library foi instalada com sucesso
            if(JFolder::exists(JPATH_LIBRARIES.'/noboss')) {
                // Recarrega a pagina
                header("Refresh:0");
                exit;
            }

            // Library ainda nao foi instalada: exibe erro na tela
            $msg = "There was an error during the installation or update of No Boss Library, required for the operation of all No Boss extensions. To solve the problem, follow the steps below: <br />
            * Install the library on your Joomla site using the https://www.nobossextensions.com/en/installation/nobosslibrary/ link. <br />
            * Notify No Boss support on the problem through the contact form https://www.nobossextensions.com/en/.";
            JLog::add($msg, JLog::WARNING, 'jerror');
            return;
        }

        // No Boss Ajax nao foi instalado
        if(!$nobossAjaxFolderExists) {
            $msg = "There was an error during the installation or update of No Boss Ajax, required for the operation of all No Boss extensions. To solve the problem, follow the steps below: <br />
            * Install the No Boss Ajax on your Joomla site using the https://www.nobossextensions.com/en/installation/nobossajax/ link. <br />
            * Notify No Boss support on the problem through the contact form https://www.nobossextensions.com/en/.";
            JLog::add($msg, JLog::WARNING, 'jerror');
            return;
        }
    }

    /*
        TODO: funcao temporaria para atualizar novo formato de cadastro de horarios para os eventos
                * Executado a partir da versao 2.1 criada em Abril de 2020
                * Remover apos Abril de 2021

            Observacoes:
                * Na extensao header, tem exemplos de como migrar colunas de uma modal para outra
    */
    function updateVersion2_1(){
        $db = JFactory::getDbo();

        /* TODO: exemplo de como testar a funcao a partir de uma chamada fixa no arquivo principal do modulo:
                require_once 'script.extension.php';
                mod_NobosscalendarInstallerScript::updateVersion2_1();
        */ 

        // INICIO: VERIFICA SE NOVA COLUNA JA EXISTE NO BANCO ANTES DE SEGUIR COM ATUALIZACAO
        
        $query = $db->getQuery(true);
        $config = JFactory::getConfig();
        
        // Prefixo da tabela
        $dbprefix = $config->get('dbprefix');
        
        // Consulta que verifica se tabela existe no banco
        $query = "SELECT COUNT(*) FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME='{$dbprefix}noboss_calendar' AND COLUMN_NAME = 'hours'";
       
        $db->setQuery($query);
        $count = $db->loadResult();
        
        // Coluna ja existe no banco: nao ha atualizacoes a realizar
        if ($count > 0){
            return;
        }

        // INICIO: CRIA NOVA COLUNA NO BANCO
        $query = $db->getQuery(true);
        $query = "ALTER TABLE  `#__noboss_calendar` ADD COLUMN `hours` text AFTER `modal_content`,
        ADD COLUMN `input_type` varchar(30) DEFAULT 'manual' AFTER `hours`;";
        $db->setQuery($query);
        $db->execute();

        // INICIO: MIGRA CAMPO DE HORAS PARA COLUNA PROPRIA

        $query = $db->getQuery(true);

        $query
        ->select("id_event, data_json")
        ->from('#__noboss_calendar');

        // Seta query
        $db->setQuery($query);
        
        //echo str_replace('#__', 'ext_', $query); exit;

        try{
            $modules = $db->loadObjectList();

            if(count($modules) > 0){
                // Percorre cada modulo a ser atualizado
                foreach ($modules as $module) {
                    $params = json_decode($module->data_json);

                    // Data inicial definida
                    if(!empty($params->initial_time)){
                        // Gera array de objetos para novo campo do banco
                        $hours = array();
                        $hour = new stdClass();
                        $hour->initial = $params->initial_time;
                        $hour->final = $params->final_time;
                        $hours[] = $hour;
                        
                        // Converte para json para salvar no banco
                        $hours = json_encode($hours);

                        // Adiciona contrabarra nas aspas duplas
                        $hours = str_replace('"', '\\"', $hours);
                    }
                    else{
                        $hours = '';
                    }
                    
                    // Remove campos do local de origem
                    unset($params->initial_time);
                    unset($params->final_time);

                    // Codifica novamente para salvar
                    $params = json_encode($params);
    
                    $query = $db->getQuery(true);

                    // Realiza update no banco
                    $query->update($db->quoteName('#__noboss_calendar'))
                            ->set($db->quoteName('hours') . ' = ' . $db->quote($hours))
                            ->set($db->quoteName('data_json') . ' = ' . $db->quote($params))
                            ->where("id_event = '{$module->id_event}'");

                    //echo str_replace('#__', 'ext_', $query); exit;

                    // Seta query
                    $db->setQuery($query);
                    $db->execute();

                }

            }
        }
        catch(Exception $e){
            throw new Exception('Erro ao tentar converter os registros da versao antiga para nova');
        }
    }
}
