<?php
/**
 * @package			No Boss Extensions
 * @subpackage  	No Boss Library
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2020 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

defined('JPATH_PLATFORM') or die;

jimport('noboss.util.curl');

// Classe para requisicoes para API do Facebook
class NoBossApiFacebook {

    private $redirectURI;
    private $client;
    private $config;
    private $scope = "user_videos";
    private $oauthURL = 'https://www.facebook.com/v7.0/dialog/oauth';
    private $oauthTokenURL = 'https://graph.facebook.com/v7.0/oauth/access_token';

    /**
     * __construct
     *
     * @param   string      $redirectURI    url para redirecionar apos autorizar aplicacao (deve estar tb cadastrada no console da API)
     * @param   array       $config         matriz associativa, deve conter client_id e client_secret.
     * 
     * @return void
     */
    public function __construct($redirectURI, $config) {
        $this->redirectURI = $redirectURI;
        $this->config = $config;
    }

    /**
     * Autoriza o usuario junto a API obtendo o token para executar proximos metodos
     *
     * @return string|null access_token needed for other methods
     */
    public function authorize() {
        if (!isset($_GET['code'])) {
            $authURL = "{$this->oauthURL}?" . \http_build_query([
            'response_type' => 'code',
            'client_id' => $this->config['client_id'],
            'redirect_uri' => $this->redirectURI,
            'state' => '1',
            'scope' => $this->scope
            ]);
            header('Location: ' . filter_var($authURL, FILTER_SANITIZE_URL));
        } else {
            $authCode = $_GET['code'];
            $accessToken = $this->authenticate($authCode)['access_token'];
            return $accessToken;
        }
    }

     /**
     * Autentique o codigo de troca para a estrutura access_code.
     *
     * @param   string  $code   returned by authorize()
     * 
     * @return  array access and refresh token
     */
    private function authenticate($code) {
        $queryParams = [
        'client_id' => $this->config['client_id'],
        'redirect_uri' => $this->redirectURI,
        'client_secret' => $this->config['client_secret'],
        'code' => $code
        ];
        
        $response = NoBossUtilCurl::request("GET", "{$this->oauthTokenURL}?" . \http_build_query($queryParams));
        
        if (!$response->success) {
            $data = json_decode($response->data);
            throw new \Exception($data->error->message, $data->error->code);
        }
            
        return \json_decode($response->data, true);
    }

    /**
     * Obtem todos videos de um usuario
     *
     * @param   string      $token      access_token
     * @param   string      $userId     id do usuaqio
     * 
     * @return  array       videos do usuario
     */
    // FIXME: TERMINAR DE DESENVOLVER A FUNCAO - TEMOS QUE VER COMO OBTER O ID DO USUARIO QUE LOGOU
    public function getVideosUser($token, $userId) {
        $dest = "https://graph.facebook.com/v7.0/{$userId}/videos";

        $queryParams = [
            //'fields' => 'description,title,created_time,thumbnails',
            'access_token' => $token
        ];

        $headers = [
        'Accept' => 'application/json'
        ];
        
        $response = NoBossUtilCurl::request("GET", "{$dest}?" . \http_build_query($queryParams), null, $headers);
        
        if (!$response->success) {
            $data = json_decode($response->data);
            throw new \Exception($data->error->message, $data->error->code);
        }

        $json = \json_decode($response->data, true);
        
        return $json;
    }

    /**
     * Obtem videos de uma pagina
     *
     * @param   string      $token      access_token
     * @param   string      $pageId     id da pagina
     * 
     * @return  array       videos do usuario
     */
    // FIXME: TERMINAR DE DESENVOLVER A FUNCAO - TEMOS QUE VER COMO OBTER O ID DA PAGINA. SERIA NECESSARIO ANTES TER OUTRA FUNCAO QUE BUSQUE TODAS FAGINAS VINCULADAS A UM USUARIO
    public function getVideosPage($token, $pageId) {
        $dest = "https://graph.facebook.com/v7.0/{$pageId}/videos";

        $queryParams = [
            //'fields' => 'description,title,created_time,thumbnails',
            'access_token' => $token
        ];

        $headers = [
        'Accept' => 'application/json'
        ];
        
        $response = NoBossUtilCurl::request("GET", "{$dest}?" . \http_build_query($queryParams), null, $headers);
        
        if (!$response->success) {
            $data = json_decode($response->data);
            throw new \Exception($data->error->message, $data->error->code);
        }

        $json = \json_decode($response->data, true);
        
        return $json;
    }

    /**
     * Obtem detalhes de um video (permite somente videos do proprio usuario)
     *
     * @param   string      $token access_token
     * @param   string      $videoID id of video
     * 
     * @return  array       dados do video
     */
    public function getVideoData($token, $videoID) {
        $dest = "https://graph.facebook.com/v7.0/{$videoID}";

        $queryParams = [
            // FIXME: ver se quero manter limitacao dos campos retornados
            'fields' => 'description,title,created_time,thumbnails',
            'access_token' => $token
        ];

        $headers = [
        'Accept' => 'application/json'
        ];
        
        $response = NoBossUtilCurl::request("GET", "{$dest}?" . \http_build_query($queryParams), null, $headers);
        
        if (!$response->success) {
            $data = json_decode($response->data);
            throw new \Exception($data->error->message, $data->error->code);
        }

        $json = \json_decode($response->data, true);
        
        return $json;
    }
}
