<?php
/**
 * @package			No Boss Extensions
 * @subpackage  	No Boss Library
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2020 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

defined('JPATH_PLATFORM') or die;

// Classe para traducoes via google translate
class NoBossApiGoogletranstale {
   
    /**
	 *  Funcao principal para salvar os dados da planilha importada
	 *
     * @param   Mides       $text           Texto a ser traduzido (string se for apenas um texto ou array se forem varios textos separados)
     * @param   String      $source         Tag do idioma de origem (ex: 'pt')
     * @param   String      $target         Tag do idioma a ser traduzido (ex: 'en')
     * @param   String      $apiKey         Chave da API do Google
     * @param   String      $format         Formato de retorno dos dados (valores possíveis: 'text' ou 'html')
     * @param   Array       $replaces       Array com valores que deseja fazer replace apos traducao 
     *                                        * Util para casos em que o google faz alguma traducao errada ou quebra uma tag html
     *                                        * Exemplo de como passar valors: 
     *                                              array(array('search' => 'texto origem', 'replace' => 'texto modificado'));
     * 
	 * @return  mixed       String com texto traduzido (qnd enviado apenas string única) ou array com traducoes
	 */
    public function gooogleTranslate($text, $source, $target, $apiKey, $format = 'text', $replaces = array()){
        $queryTexts = '';

        // Recebido um array de itens a traduzir
        if(is_array($text)){
            foreach ($text as $q) {
                $queryTexts .= "&q=".rawurlencode($q);
            }
        }
        // Recebida uma string a traduzir
        else{
            $queryTexts .= "&q=".rawurlencode($text);
        }

        // Url para requisicao
        $url = "https://www.googleapis.com/language/translate/v2?key={$apiKey}{$queryTexts}&target={$target}&source={$source}&format={$format}";

        // Realiza requisicao simples via curl
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);                 
        curl_close($ch);

        $result = json_decode($response,true);

        if($result != null) {
            // Ocorreu um erro
            if(isset($result['error'])) {
                throw new RuntimeException("Erro: ".$result['error']['message']);
            }
            // Traducao teve sucesso
            else{
                $translated = array();
                // Percorre cada traducao solicitada
                foreach ($result['data']['translations'] as $translate) {
                    // Definido replaces a fazer no texto traduzido
                    if(count($replaces) > 0){
                        // Percorre cada replace a executar
                        foreach ($replaces as $item) {
                            $translate['translatedText'] = str_replace($item['search'], $item['replace'], $translate['translatedText']);
                        }
                    }

                    // Guarda item traduzido
                    $translated[] = $translate['translatedText'];
                }

                // Existe mais de uma posicao traduzida no array: retorna array
                if(count($translated) > 1){
                    return $translated;
                }
                // Existe somente uma posicao de array traduzida: retorna direto a string
                else{
                    return $translated[0];
                }
            }
        }
       
        return false;
    }

}
