<?php
/**
 * @package			No Boss Extensions
 * @subpackage  	No Boss Library
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2020 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/**
 * Script de instalação do pacote.
 * As extensões são instaladas seguindo a própria rotina do joomla, o pacote somente instala os extras na pasta 'packages'
 */
class pkg_NobossInstallerScript{
    /**
    * A versão minima do php para instalar essa extensão
    * @var   string
    */
    protected $minPHPVersion = '5.3.17';
    /**
    * A versão minima do Joomla para instalar essa extensão
    * @var   string
    */
    protected $minJoomlaVersion = '3.5.0';

    /**
     * Evento pre-fligt do Joomla. Esse método é executado antes de qualquer outro, sendo instalação ou update
     * Esse é o unico momento onde pode cancelar a instalação
     *
     * @param   string     $type   Tipo de intalações (install, update, discover_install)
     * @param   JInstaller $parent Parent object
     *
     * @return  boolean  true caso deva ser instalada, false para cancelar a intalação
     */
    function preflight($type, $parent){
        // Verifica a versão minima do php
        if (!empty($this->minPHPVersion)){
            // Verifica a versão do php
            if (function_exists('phpversion')){
                $version = phpversion();
            }else{
                $version = '5.0.0';
            }
            // Verifica versão do php
            if (version_compare($version, $this->minPHPVersion, '<')){
                $msg = JText::sprintf("SCRIPT_INSTALLATION_MSG_PHP_VERSION", $this->minPHPVersion, $version);
                JLog::add($msg, JLog::ERROR, 'jerror');
                throw new RuntimeException($msg, 500);
            }
        }
        // Verifica a versão minima joomla
        if (!empty($this->minJoomlaVersion) && version_compare(JVERSION, $this->minJoomlaVersion, '<')){
            $jVersion = JVERSION;
            $msg = JText::sprintf("SCRIPT_INSTALLATION_MSG_JOOMLA_VERSION", $this->minJoomlaVersion, $jVersion);
            throw new RuntimeException($msg, 500);
        }

        // Percorre todos os zip da pasta packages e e instala um por um
        $this->installPackageExtras($parent);
    }

    /**
     * Method to run after an install/update/uninstall method
     *
     * @param   \JInstallerAdapterPackage   $parent is the class calling this method
     * @param  	String  $type is the type of change (install, update or discover_install)
     *
     * @return void
     */
    function postflight($type, $parent){
        $db = JFactory::getDbo();

        $newPackageId = 0;
        
        $zipFiles = JFolder::files(__DIR__, '.*\.zip$');
        // Pega o arquivo 
        $zipFile = reset($zipFiles);
        // Verifica se foi instalado o p
        if(substr($zipFile, 0, 3) == 'pkg'){
            $element = explode('.', $zipFile);
            $element = $element[0];
            $query = $db->getQuery(true);
            $query->select("a.extension_id")
                    ->from("#__extensions as a")
                    ->where("a.element = '{$element}'");
            $db->setQuery($query);
            $newPackageId = $db->loadResult();
        }

        // Atualiza as extensões para não estarem relacionadas ao pacote
        $query = $db->getQuery(true);
        $query->update("#__extensions as a, (SELECT b.extension_id FROM #__extensions as b WHERE b.element = 'pkg_noboss') as b")
            ->set("a.package_id = '{$newPackageId}'")
            ->where("a.package_id = b.extension_id");
        $db->setQuery($query);
        $result = $db->execute();

        // Deleta a linha do pacote no banco
        $query = $db->getQuery(true);
        $query->delete('#__extensions')
            ->where("element = 'pkg_noboss'");
        $db->setQuery($query);
        $result = $db->execute();
    }

//************************************************************************************************************
//*************************************** NÃO EDITE DEPOIS DESSA LINHA ***************************************
//************************************************************************************************************

    private function getLibraryUpdateId($parent){
        // Pega o alias da library
        $libElement = 'noboss';

        // Faz uma busca no  banco, na tabela extensions
        $db = JFactory::getDbo();
        $query = $db->getQuery(true);

        //Monta a query para buscar na tabella extension
        $query->select("b.update_id")
                ->from("#__extensions as a")
                ->join('INNER',"#__updates as b ON a.extension_id = b.extension_id")
                ->where("a.element = 'noboss'");
        $db->setQuery($query);
        $result = $db->loadResult();

        return (int) $result;
    }

    /**
     * Percorre todas as pastas dentro da pasta 'packages' e executa a instalação de cada uma
     *
     * @param Object $parent
     * @return void
     */
    private function installPackageExtras($parent){
        // Lê as pastas dentro da pasta packages
        $packages = JFolder::folders(__DIR__ . '/packages');
        if(!$packages){
            return;
        }
        // Para cada uma delas, faz a instalação do joomla
        foreach ($packages as $package){
            $tmpInstaller = new JInstaller;
            try{
                $tmpInstaller->setPath('source', __DIR__ . '/packages/' . $package);
                $tmpInstaller->findManifest();
                $manifest = $tmpInstaller->getPath('manifest');

                $manifestPath = \JPath::clean($manifest);
                $element = preg_replace('/\.xml/', '', basename($manifestPath));
                $manifest = $tmpInstaller->getManifest();
                // Ajusta o element de acordo com o timpo de extensão
                if($manifest['type'] == 'component'){
                    if (strpos($element, 'com_') !== 0){
                        $element = 'com_' . $element;
                    }
                } else if ($manifest['type'] == 'plugin'){
                    if (strpos($element, 'plg_') !== 0){
                        $element = 'plg_' . $element;
                    }
                } else if ($manifest['type'] == 'package'){
                    if (strpos($element, 'pkg_') !== 0){
                        $element = 'pkg_' . $element;
                    }
                } else if ($manifest['type'] == 'module'){
                    if (strpos($element, 'mod_') !== 0){
                        $element = 'mod_' . $element;
                    }
                }

                $extension = $this->getExtensionByElement($element);
                if(!empty($extension)){
                    $extension = json_decode($extension->manifest_cache);

                    $installResult = false;
                    if(!empty($manifest)){
                        if(version_compare($manifest->version, $extension->version, '>')){
                            // Executa a instalação da pasta
                            $installResult = $tmpInstaller->install(__DIR__ . '/packages/' . $package);
                        }
                    }
                } else {
                    $installResult = $tmpInstaller->install(__DIR__ . '/packages/' . $package);
                }
            } catch (\Exception $e){
                $installResult = false;
                $error = $e->getMessage();
                JLog::add(JText::sprintf('SCRIPT_INSTALLATION_EXTRA_ERROR_EXCEPTION', $package), JLog::ERROR);
            }
            // Verifica se a instalação retornou false ou true
            if($installResult){
                JLog::add(JText::sprintf('SCRIPT_INSTALLATION_EXTRA_ERROR_INSTALLED', $package), JLog::INFO);
            } else {
                JLog::add(JText::sprintf('SCRIPT_INSTALLATION_EXTRA_ERROR_NO_INSTALLED', $package), JLog::ERROR);
            }
        }
    }

    private function getExtensionByElement($element){
        $db = JFactory::getDbo();

        $query = $db->getQuery(true);
        $query->select('*')
            ->from('#__extensions')
            ->where("element = '{$element}'")
            ->limit(1);
        $db->setQuery($query);

        $result = $db->loadObject();
        return $result;
    }

}
